      PROGRAM FLUXCHK
C stripped down version of FLUX to check the input, and
C to compare the specified stopping parameters to the Ziegler value.
C
C ===---===---===---===---===---===---===---===--- MAIN PROGRAM 
C
C INCLUDE FILE FLUX7.COM
      IMPLICIT NONE
      INCLUDE 'FLUX7.COM'

      INTEGER KL,IL
      INTEGER ISTOP
      REAL COLOSS, ATOVOL, ZILOSS
      COMMON /ELOSCOM/ KL,IL,ISTOP,COLOSS(ML), ATOVOL(ML),
     +                 ZILOSS(ML,NSTOPMAX)

      INTEGER ier, K

C SET THE VERSION STRING:
      VERSION(1:5)='V7.8'
      VERSION(16:28)=' FLUXCHK2000'

      LUN(2) = 0
      LUN(3) = 0

      CALL BATINIT
      CALL INP(INLU,ier)
      if(ier.ne.0)STOP 'error(s) in INP'
      CHECKONLY=1
      CALL PRELIMI
      CALL OUTP1
      do 24 K=1,NLAY
      CALL STOPSIM(K)
   24 CONTINUE
      END
C -------------------------------------------------- FUNCTION CORELOS
      FUNCTION CORELOS(S)
      IMPLICIT NONE
      REAL CORELOS
      REAL S

      INCLUDE 'FLUX7.COM'

C For given S, calculate energy loss due to core electrons.
C The result is multiplied by 2*pi*S to get a useful function
C for the integration.

      INTEGER KL,IL
      INTEGER ISTOP
      REAL COLOSS, ATOVOL, ZILOSS
      COMMON /ELOSCOM/ KL,IL,ISTOP,COLOSS(ML), ATOVOL(ML),
     +                 ZILOSS(ML,NSTOPMAX)

      INTEGER IB
      REAL DECO, FRAC

      IB=S*25.0
      DECO= 0.0
      IF(IB.LT.50)THEN
        IF(IB.EQ.0)IB=1
        FRAC=S*25.0-FLOAT(IB)
        DECO=EXP((1.0-FRAC)*ELCORE(IB,IL,KL,ISTOP) +
     +            FRAC*ELCORE(IB+1,IL,KL,ISTOP))
      ENDIF
      CORELOS= DECO*2E6*PI*S
      END
C -------------------------------------------------- SUBROUTINE STOPSIM
      SUBROUTINE STOPSIM(K)
      IMPLICIT NONE
      INTEGER K

      INCLUDE 'FLUX7.COM'

      REAL CORELOS
      EXTERNAL CORELOS

      INTEGER KL,IL
      INTEGER ISTOP
      REAL COLOSS, ATOVOL, ZILOSS
      COMMON /ELOSCOM/ KL,IL,ISTOP,COLOSS(ML), ATOVOL(ML),
     +                 ZILOSS(ML,NSTOPMAX)

C calculate the stopping power, resulting from the input parameters
C dedxvl, dedxvp, and impact parameter dependent energy loss

C also calculate the Ziegler values

      INTEGER I, KERR, ISTOP1
      REAL TOTALLOSS, RSTOPLOSS, FSTOPLOSS, HSTOPLOSS, TI
      REAL A,B,EPS,PREC,H

      KL=K

      do 478 ISTOP1=1,NSTOP
      ISTOP= ISTOP1
      TI= T0
      if(NSTOP.gt.1) TI= ESTOP(ISTOP)
      TOTALLOSS = 0.0
      RSTOPLOSS = 0.0
      FSTOPLOSS = 0.0
      HSTOPLOSS = 0.0
      DO 455 I=1,NLA(KL)
      IL=I
      ATOVOL(I)= CELL(K) / APC1(I,K)
      A=0.01
      B=2.0
      EPS= 0.0001
      PREC= 0.0001
      CALL ROMBERG(CORELOS,A,B,EPS,COLOSS(I),PREC,H,KERR)
      IF(KERR.NE.0)THEN
         write(6,*)'ROMBERG flunked with kerr=',KERR
         stop 'arggh!'
      ENDIF
      COLOSS(I)= COLOSS(I)*ATOMFRAC(I,KL)/ATOVOL(I)
      TOTALLOSS = COLOSS(I) + TOTALLOSS
      CALL STOPTAB(I)
      RSTOPLOSS = RSTOPLOSS + ZILOSS(I,1)*ATOMFRAC(I,KL)
      FSTOPLOSS = FSTOPLOSS + ZILOSS(I,2)*ATOMFRAC(I,KL)
      HSTOPLOSS = HSTOPLOSS + ZILOSS(I,3)*ATOMFRAC(I,KL)
  455 CONTINUE
      write (6,70) KL
      write (6,77) (ATOVOL(I),I=1,NLA(KL))
      write(6,76) TI
      write (6,71) TOTALLOSS,(COLOSS(I),I=1,NLA(KL))
      write (6,75) DEDXVL(KL,ISTOP) + DEDXVP(KL,ISTOP)
      TOTALLOSS = TOTALLOSS + DEDXVL(KL,ISTOP)+ DEDXVP(KL,ISTOP)
      write (6,72) TOTALLOSS
      write (6,74) TI,     RSTOPLOSS,
     +             (ZILOSS(I,1)*ATOMFRAC(I,KL),I=1,NLA(KL))
  478 continue
      if(NSTOP.gt.1) goto 480
      write (6,74) T0*0.75,FSTOPLOSS,
     +             (ZILOSS(I,2)*ATOMFRAC(I,KL),I=1,NLA(KL))
      write (6,74) T0*0.5, HSTOPLOSS,
     +             (ZILOSS(I,3)*ATOMFRAC(I,KL),I=1,NLA(KL))
  480 continue
  70  FORMAT(//'LAYER',I2,':  --- ENERGY LOSS ESTIMATES ---')
  71  FORMAT(' random stopping power in eV/A, core   ', 8F9.3)
  72  FORMAT(' random stopping power in eV/A, total  ', 8F9.3)
  75  FORMAT(' random stopping power in eV/A, valence', 8F9.3)
  74  FORMAT(/' Ziegler values T=',F12.3,' MeV     ', 8F9.3)
  76  FORMAT(' evaluated at T=',F12.3,' MeV')
  77  FORMAT(' atomic volume, in angstrom^3          ',9X, 8F9.3)
      END
C -------------------------------------------------- SUBROUTINE STOPTAB
      SUBROUTINE STOPTAB(I)
      IMPLICIT NONE
      INTEGER I

      INCLUDE 'FLUX7.COM'

      INTEGER KL,IL
      INTEGER ISTOP
      REAL COLOSS, ATOVOL, ZILOSS
      COMMON /ELOSCOM/ KL,IL,ISTOP,COLOSS(ML), ATOVOL(ML),
     +                 ZILOSS(ML,NSTOPMAX)

      REAL SE(NSTOPMAX), ATRHO, E1
      INTEGER NSTEP, ISTP

C stopping power according to tabulated value

       NSTEP=3
       SE(1)=T0*1000.0
       if(NSTOP.gt.1)SE(1)=ESTOP(ISTOP)*1000.0
       E1=SE(1)
       SE(2)=E1*0.75
       SE(3)=E1*0.5
       CALL rstop(Z1,A1,Z2(I,KL),E1,SE,NSTEP,1,88,ATRHO,A2(I,KL))
       do 173 ISTP=1,NSTEP
       ZILOSS(I,ISTP)=SE(ISTP)*10.0/ATOVOL(I) ! Why the factor 10?
  173  continue
       end
