CCC THE FOLLOWING ROUTINES ARE FOR USE OF THE HARTEE-FOCK POTENTIAL
CCC BASED ON SOLID STATE CHARGE DENSITIES TABULATED IN:
CCC J.F. Ziegler, J.P. Biersack and U. Littmark,
CCC "The Stopping and Range of Ions in Solids",
CCC (Pergamon, New York, 1985)
CCC
CCC The tables are stored in file 'HFCHARGE.DAT'
CCC See comments in that file for details how to add more Z values

CC edit aug 2018 (version 7.9.6) find density at r=0 by extrapolation  
CC set density to 0 in previous versions

      FUNCTION POT(Z)
      implicit none
      real POT, Z
CONTINUUM STRING POTENTIAL
C Z IN ANGSTROM UNITS
C The parameter IZ2 is passed via common block COMPOTZ
      include 'compotz.com'
      include 'zblcom.com'
      POT= APOLNOX(Z,R(0,IZ2),VRS(0,IZ2),4,MAXI(IZ2))/Z
      END

      FUNCTION DPOT(Z)
      implicit none
      real DPOT, Z
C MINUS DERIVATIVE OF UNIVERSAL CONTINUUM STRING POTENTIAL
C Z IN ANGSTROM UNITS
C The parameter IZ2 is passed via common block COMPOTZ
      include 'compotz.com'
      include 'zblcom.com'
      DPOT= APOLNOX(Z,R(0,IZ2),FRS(0,IZ2),4,MAXI(IZ2))/(Z*Z)
      END

      SUBROUTINE HFZBL(Z2)
      implicit none
      REAL Z2
      INTEGER MAXSIZE
      parameter(MAXSIZE=89)
      REAL RAD(0:MAXSIZE),DENS(0:MAXSIZE)
      INTEGER IZ2, IER, INTSIZE, I
      character*80 fullname
      external fullname

      real APOLNOX
      external APOLNOX

      IZ2= NINT(Z2)
      open (14, file=fullname('HFCHARGE.DAT'), status='OLD',ERR=471)
      CALL READHF(IZ2,RAD(1),DENS(1),MAXSIZE,INTSIZE,IER)
      DO 88 I=INTSIZE+1, MAXSIZE
        RAD(I)=RAD(I-1)+0.01
        DENS(I)=0.0
  88  CONTINUE
c find density at r=0 by extrapolation  
      RAD(0)=0
      DENS(0)= APOLNOX(0.0,RAD(1),DENS(1),4,MAXSIZE)
      IF(IER .EQ. 0)then 
        CALL SCREEN(RAD, DENS, Z2)
      else
        write(6,*) 'HFZBL: Sorry, Z =', Z2,' not implemented'
        stop 'illegal value of Z2'
      endif
      RETURN
  471 write(6,*)'Failure to open file '//fullname('HFCHARGE.DAT')
      STOP 'can''t find HF data file'
      END

      SUBROUTINE SCREEN(RAD,DENS,TOTCH)
      implicit none
      INTEGER MAXSIZE
      parameter(MAXSIZE=89)
      REAL RAD(0:MAXSIZE),DENS(0:MAXSIZE),TOTCH
      real QFUN,PFUN,FFUN,VFUN
      EXTERNAL QFUN,PFUN,FFUN,VFUN
      include 'compotz.com'
      include 'zblcom.com'
      integer I,IMAX,J,KERR
      real R1,R2,PREC,H
      real DQ,DP,XMAX,FRSJ,VRSJ
      Z2=TOTCH
      IZ2= NINT(Z2)
      DO 110 I=0,MAXSIZE
      R(I,IZ2)=RAD(I)
  110 D(I)=DENS(I)*RAD(I)*12.56637     
C 4*PI*R*RHO
      DO 120 I=MAXSIZE,1,-1
      IF(D(I).NE.0)GOTO 121
  120 CONTINUE
  121 MAXI(IZ2)=I
      IMAX=I
      QT(IMAX)=0
      PS(IMAX)=0
      PH(IMAX)=0
      VRS(IMAX,IZ2)=0
      FRS(IMAX,IZ2)=0
      DO 128 J=IMAX-1,0,-1
      R1=R(J,IZ2)
      R2=R(J+1,IZ2)
      PREC=(R2-R1)*0.0001
      CALL ROMBERG(QFUN,R1,R2,0.0001,DQ,PREC,H,KERR)
*     IF(KERR.NE.0)WRITE(6,125)KERR,'QFUN'
      CALL ROMBERG(PFUN,R1,R2,0.0001,DP,PREC,H,KERR)
*     IF(KERR.NE.0)WRITE(6,125)KERR,'PFUN'
* 125 FORMAT(' ROMBERG: KERR=',I5,A10)
      QT(J)=QT(J+1)+DQ
      PS(J)=PS(J+1)+DP
      PH(J)=(QT(J)-R1*PS(J))/Z2
  128 CONTINUE
      WRITE(6,130)QT(0)
  130 FORMAT(' HF_ZBL: TOTAL CHARGE=',F9.4)
* uncomment the following to obtain test output  
* 140 FORMAT(A40/(8(F9.5,1X)))
*     WRITE(6,140) 'ATOMIC RADIUS (ANGSTROM)',RAD
*     WRITE(6,140) 'SCREENING FUNCTION (FROM CHARGE DENSITY)',PH
      DO 170 J=1,IMAX-1
      RH=R(J,IZ2)
      XMAX=SQRT(R(IMAX,IZ2)**2-RH**2)
      PREC=XMAX*0.00001
      CALL ROMBERG(FFUN,0.0,XMAX,0.00001,FRSJ,PREC,H,KERR)
*     IF(KERR.NE.0)WRITE(6,125)KERR,'FFUN'
      CALL ROMBERG(VFUN,0.0,XMAX,0.00001,VRSJ,PREC,H,KERR)
*     IF(KERR.NE.0)WRITE(6,125)KERR,'VFUN'
      FRS(J,IZ2)=RH**3*FRSJ*2.0/Z2
      VRS(J,IZ2)=RH*VRSJ*2.0
  170 CONTINUE
* uncomment the following to obtain test output  
*     WRITE(6,140) 'CONTINUUM STRING POTENTIAL (STATIC)',
*    + (VRS(J,IZ2),J=0,MAXSIZE)
*     WRITE(6,140) 'CONTINUUM STRING FORCE (STATIC)',
*    + (FRS(J,IZ2),J=0,MAXSIZE)
      END

      FUNCTION QFUN(X)
      implicit none
      real QFUN,X
      include 'compotz.com'
      include 'zblcom.com'
      QFUN= APOLNOX(X,R(0,IZ2),D(0),4,MAXI(IZ2))*X
      END

      FUNCTION PFUN(X)
      implicit none
      real PFUN,X
      include 'compotz.com'
      include 'zblcom.com'
      PFUN= APOLNOX(X,R(0,IZ2),D(0),4,MAXI(IZ2))
      END

      FUNCTION FFUN(X)
      implicit none
      real FFUN,X
      include 'compotz.com'
      include 'zblcom.com'
      real U
      U=SQRT(RH**2+X**2)
      FFUN= APOLNOX(U,R(0,IZ2),QT(0),4,MAXI(IZ2))/U**3
      END

      FUNCTION VFUN(X)
      implicit none
      real VFUN,X
      include 'compotz.com'
      include 'zblcom.com'
      real U
      U=SQRT(RH**2+X**2)
      VFUN= APOLNOX(U,R(0,IZ2),PH(0),4,MAXI(IZ2))/U
      END

C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FUNCTION APOLNOX(A,X,Y,N,MAXI)
C special version of APOL ( NO EXTRAPOLNOXATION BEYOND X(MAXI) )
C     APOLNOX FINDS THE VALUE OF Y FOR X=A BY N-POINTS INTERPOLATION
C     FROM THE ARRAYS X AND Y, BOTH WITH SIZE MAXI.  N MAY BE 2 TO 6.
      REAL FUNCTION APOLNOX (A,X,Y,N,MAXI)
      IMPLICIT NONE
C usage: R=APOLNOX(A,X,Y,N,MAXI)
      INTEGER N, MAXI
      REAL A, X(MAXI),Y(MAXI)

      REAL PX(6),PY(6)
      INTEGER I1,I2,I,K
      REAL S

C     FIND I1 AND I2 SUCH THAT A IS BETWEEN X(I1) AND X(I2)
      I1=1
      I2=MAXI

      IF(X(MAXI).GT.X(1))THEN
        S=1.0
      ELSE
        S=-1.0
      ENDIF

***** NO EXTRAPOLATION BEYOND X(MAXI)  !! (SPECIAL VERSION)
      IF((A-X(MAXI))*S.GT.0)THEN
          APOLNOX=0
          RETURN
      ENDIF
*****

1     I=(I1+I2)/2
      IF ((A - X(I))*S .GT. 0) THEN
        I1=I
      ELSE
        I2=I
      ENDIF
      IF (I2-I1 .GT. 1) GOTO 1

C     NOW,SET I1 TO INDEX-1 OF FIRST ELEMENTS TO BE USED
      I1=I1-N/2
      IF(I1 .LT. 0) I1=0
      IF (I1+N .GT. MAXI) I1=MAXI-N
C     TRANSFER  A-X AND Y TO AUXILIARY ARRAYS
      DO 5 I=1,N
      I1=I1+1
      PX(I)=A-X(I1)
5     PY(I)=Y(I1)
C   FINALLY, USE AITKEN ALGORITHM FOR LAGRANGE INTERPOLATION
      DO 6 K=1,N-1
      DO 6 I=K+1,N
6     PY(I)=(PY(I)*PX(K)-PY(K)*PX(I))/(PX(K)-PX(I))
      APOLNOX=PY(N)
      END

C ---------------------------------------------- subroutine READHF
      SUBROUTINE READHF(IZ2,ARRAY1,ARRAY2,MAXSIZE,INTSIZE,IER)
C subroutine READHF reads an array of real numbers from a file
C assigned to logical unit 14.
C The file is assumed to have the following structure:
C   **name1
C   Ndata, Z2
C   Ndata pairs of real numbers (radius, charge density)
C   **name2
C   etcetera
C Where name1, name2,.. are identifiers of subfiles.
C  Each name must be preceeded in the file by ** in column 1.
C
C CALLING SEQUENCE
C     CALL READHF(IZ2,ARRAY1,ARRAY2,MAXSIZE,INTSIZE,IER)
C where
C     IZ2     = Z2 (input).
C     ARRAY1,2= the arrays to store the data to be read
C     MAXSIZE = the maximum number of data allowed (input)
C     INTSIZE = the actual number of data read (i.e. Ndata), (output)
C
C Error exits occur when the subfile is not found, or if MAXSIZE is
C exceeded.
C
      PARAMETER (LU=14)
      REAL ARRAY1(MAXSIZE), ARRAY2(MAXSIZE)
      CHARACTER*80 BUFFER
      REWIND(LU)
      SIZE=0
 111  CONTINUE
      READ(LU,222,ERR=331,END=333)BUFFER
      if(BUFFER(1:2) .ne. '**')goto 111
 222  FORMAT(A80)
C NAME FOUND, READ SIZE AND AUXNUM
      ZREAD=0
      READ(LU,*)NREAD, IZREAD
      IF(IZREAD .NE. IZ2) goto 111
      INTSIZE=NREAD
      IF(INTSIZE.GT.MAXSIZE)goto 334
      READ(LU,*,ERR=331,END=331)(ARRAY1(I),ARRAY2(I),I=1,INTSIZE)
      IER=0
      RETURN
 331  WRITE(6,*)'read error'
      IER= -2
      RETURN
 333  WRITE(6,*)'subfile not found.'
      IER= -1
      RETURN
 334  write(6,*)
     + 'subfile exceeds max size, see parameter MAXSIZE in file hf.f'
      IER= +1
      RETURN
      END
