      PROGRAM PLANPOT
      implicit NONE

* CONTINUUM PLANAR POTENTIAL [ MOLIERE,ZBL, LINDHARD, HF ],
* AND CORRESPONDING FORCE.
* THERMALLY MODIFIED.
* OPTION 1: CALCULATE & PRINT POTENTIAL AND FORCE FOR EACH MODEL
*           (IN eV, resp. eV/angstrom), for as a function of X.
* OPTION 2: calculate & print the half wave length of flux oscillations
*           as a function of the oscillation amplitude. (in nm).
*           a properly weighted average of these should correspond
*           to the distance between wiggles in a RBS spectrum.
*
      REAL ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0
      COMMON /COM/ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0

      include 'compotz.com'

      EXTERNAL VA,VAB,VAL,VAH
      REAL ZZE2D, AVXSQ
      EXTERNAL ZZE2D, AVXSQ

      INTEGER J
      LOGICAL SWITCH
      REAL XX(0:100),VV(0:100),FF(0:100)
      REAL Z1, A1, A2
      REAL AVOL, DEBYE, TEMP
      REAL EPS, PI, AMU
      REAL pc, beta, ADENS, ZZAM, PSIA, rc, ZZAB, ZZAL, ZZAH, U1SQ
      REAL S, DEFAULTOPT, X1, X2, DD, AMPL, XM0
      REAL POTM, DPOTM, POTB, DPOTB, POTL, DPOTL, POTH, DPOTH
      REAL ALAMM, ALAMB, ALAML, ALAMH

      DATA AVOL, d1,d2,DEBYE,TEMP/20.0, 2.3517, 0.7839, 500.0, 295.0/
      DATA E0, Z1, A1, Z2, A2/ 3.0, 1.0, 1.0, 14.0, 28.0/
      DATA EPS/ 0.03/

      PI=ATAN(1.0)*4.0
      AMU=931.5016

  111 continue
      call ask(E0, 'beam energy (MeV)')
      call ask(Z1, 'Z1')
      call ask(A1, 'A1')
      call ask(Z2, 'Z2')
      call ask (A2,'A2')
      call ask(AVOL, 'atomic volume (Angst**3)')
* note. two alternating spacings between planes are allowed.
      call ask(d1,'(FIRST) INTERPLANAR DISTANCE')
      call ask(d2,'(SECOND) INTERP. DISTANCE OR 0')

C PLANAR DENSITY OF ATOMS in angstrom**(-2)
      IF(d2.EQ.0)THEN
          ADENS=d1/AVOL
      ELSE
          ADENS=0.5*(d1+d2)/AVOL
      ENDIF

C FIRSOV's screening radius, used with the MOLIERE potential
      ATF=0.4685/ (SQRT(Z1) + SQRT(Z2)) ** (2.0/3.0)
C ZBL screening radius:
      ATB=0.4685/(Z1**0.23 + Z2**0.23)
C LINDHARD:
      ATL=0.4685/SQRT(Z1**(2.0/3.0) + Z2**(2.0/3.0))
C Z1*Z2*e**2/a in MeV:
      ZZAM=ZZE2D(Z1,Z2,ATF)
      ZZAB=ZZE2D(Z1,Z2,ATB)
      ZZAL=ZZE2D(Z1,Z2,ATL)
      ZZAH=ZZE2D(Z1,Z2,1.0)
      PI=ATAN(1.0)*4.0
      WRITE(6,22) ATF,ATB,ATL,ZZAM,ZZAB,ZZAL,ADENS
 22   FORMAT('SCREENING RADIUS',/'  MOLIERE:',F8.5,/'      ZBL:',F8.5,
     +  /' LINDHARD:',F8.5/
     + 'Z1Z2/e^2a',/'  MOLIERE:',E13.5,/'      ZBL:',E13.5,
     + /' LINDHARD:',E13.5/
     + 'AREAL DENSITY: ',F9.6)
      FACM=PI*ADENS*ATF*ZZAM
      FACB=PI*ADENS*ATB*ZZAB
      FACL=PI*ADENS*ATL*ZZAL
      FACH=PI*ADENS*ZZAH

C characterictic angle
C ====================
      write(6,'(/)')
      write(6,17)'Firsov screening radius (angstrom):', ATF
      write(6,17)'Z1.Z2.e^2/a (eV):',ZZAM*1.0E6
      write(6,17)'areal density (atoms / sq. angstrom):', ADENS
      PSIA= sqrt(2.0*FACM*ATF/E0)
      WRITE(6,24) 'characteristic angle (deg):',PSIA*180.0/PI, PSIA*1E6
 24   FORMAT(A41,F12.6,' = ',F12.6,' murad')

C critical bending radius
C =======================
calculate linear momentum in units MeV/c (relativistically correct)
      pc = sqrt( (2 * A1*AMU + E0) * E0)
calculate beta = v/c
      beta = pc / ( A1*AMU + E0)
      rc = pc * beta * AVOL / (3.1415 * ZZE2D(Z1,Z2,d1) * d1 * d1)
      write(6,'(/)')
  12  format(A56, F12.3, ' micrometer')
      write(6,12)
     + 'critical bending radius     (relativistic calculation)',
     + rc*1.0E-4
      write(6,17) 'v/c=',beta
      write(6,17)'p (Mev/c) =',pc

  17  format(A56,F12.3)
      CALL ASK (DEBYE,'DEBYE TEMP(K)')
      XM=0.5*d1

      DEFAULTOPT= 1.0
* choose the option wanted:
  61  CONTINUE
      S=DEFAULTOPT
      CALL ASK(S,'POTENTIAL & FORCE=1/WIGGLES=2/RESTART=0/STOP=-1')
      DEFAULTOPT= -1.0
      SWITCH = S.GT.1.5
      IF(S.EQ.0)GOTO 111
      IF(S.LT.0)STOP

      CALL ASK (TEMP,'TEMPERATURE(K)[0=static!]')
      IF(TEMP.LE.0)THEN
          U1=0
      ELSE
          U1SQ=AVXSQ(A2,DEBYE,TEMP)
          U1 = SQRT ( U1SQ )
          WRITE(6,54)U1
  54      FORMAT('THERMAL VIBRATION AMPLITUDE U1:',F8.5)
      ENDIF
      IF(SWITCH)GOTO 41

* Potentials as a function of x
      X1=0.0
      X2=d1+d2
      CALL ASK(X1,'First value of x')
      CALL ASK(X2,'Last value of x')
      DD=(X2-X1)/100.0
      DO 51 J=0,100
      XX(J)=DD*J+X1
      CALL FPLTOT(XX(J)/ATF,U1/ATF,d1/ATF,d2/ATF,POTM,DPOTM,1)
      VV(J)=ATF*FACM*POTM*1.0E6
      FF(J)=    FACM*DPOTM*1.0E6
  51  CONTINUE
      WRITE(6,401)'DISTANCES USED', (XX(J), J=0,100)
      WRITE(6,401)'MOLIERE POTENTIAL',(VV(J),J=0,100)
     + , 'MOLIERE FORCE',FF
      DO 52 J=0,100
      XX(J)=DD*J
      CALL FPLTOT(XX(J)/ATB,U1/ATB,d1/ATB,d2/ATB,POTB,DPOTB,2)
      VV(J)=ATB*FACB*POTB*1.0E6
      FF(J)=    FACB*DPOTB*1.0E6
  52  CONTINUE
      WRITE(6,401)'ZBL POTENTIAL',(VV(J),J=0,100)
     + , 'ZBL FORCE',FF
      IF(U1.GT.0)GOTO 61 !LINDHARD THERMALLY MOD. NOT IMPLEMENTED
      DO 53 J=0,100
      XX(J)=DD*J
      CALL FPLTOT(XX(J)/ATL,U1/ATL,d1/ATL,d2/ATL,POTL,DPOTL,3)
      VV(J)=ATL*FACL*POTL*1.0E6
      FF(J)=    FACL*DPOTL*1.0E6
  53  CONTINUE
      WRITE(6,401)'LINDHARD POTENTIAL',(VV(J),J=0,100)
     + , 'LINDHARD FORCE',FF
      IZ2= NINT(Z2)
      CALL HFZBL(Z2)
      DO 55 J=0,100
      XX(J)=DD*J
      CALL FPLTOT(XX(J),U1,d1,d2,POTL,DPOTL,4)
      VV(J)=FACH*POTL*1.0E6
      FF(J)=FACH*DPOTL*1.0E6
  55  CONTINUE
      WRITE(6,401)'HF POTENTIAL',(VV(J),J=0,100)
     + , 'HF FORCE',FF
 401  FORMAT(2(/A/12(8F11.4/),5F11.4))
      GOTO 61

* Planar oscillation half-wave length as a function of amplitude
  41  CONTINUE
* ASK INTEGRATION STEP. (SAY 0.03)
      CALL ASK(EPS,'EPS')
      IZ2= NINT(Z2)
      CALL HFZBL(Z2)
      WRITE(6,64) 'HALF-WAVELENGTH'
  64  FORMAT(18X, A18)
      WRITE(6,62)'AMPL','MOLIERE','ZBL','LINDHARD','HF'
  62  FORMAT(5(A9,1X))
      DO 66 J=0,20
      AMPL=(0.96*J + 0.4)*XM/20
  63  XM0=XM-AMPL
      CALL FPLTOT(XM0/ATF,U1/ATF,d1/ATF,d2/ATF,POTM,DPOTM,1)
      CALL FPLTOT(XM0/ATB,U1/ATB,d1/ATB,d2/ATB,POTB,DPOTB,2)
      CALL FPLTOT(XM0/ATL,U1/ATL,d1/ATL,d2/ATL,POTL,DPOTL,3)
      CALL FPLTOT(XM0,U1,d1,d2,POTH,DPOTH,4)
      VM0=ATF*FACM*POTM
      VB0=ATB*FACB*POTB
      VL0=ATL*FACL*POTL
      VH0=    FACH*POTH
      CALL INTEG(XM0,XM,ALAMM,EPS,VA)
      CALL INTEG(XM0,XM,ALAMB,EPS,VAB)
      IF(U1.LE.0)THEN
        CALL INTEG(XM0,XM,ALAML,EPS,VAL)
          CALL INTEG(XM0,XM,ALAMH,EPS,VAH)
      ELSE
        ALAML=0
        ALAMH=0
      ENDIF
      ALAMM=0.2*ALAMM
      ALAMB=0.2*ALAMB
      ALAML=0.2*ALAML
      ALAMH=0.2*ALAMH
      WRITE(6,71) XM-XM0,ALAMM,ALAMB,ALAML,ALAMH
  71  FORMAT(F8.3,6X,5F9.3)
66    CONTINUE
      GOTO 61
      END

      SUBROUTINE INTEG(X0,X1,T,EPS,VA)
      implicit NONE
      REAL X0,X1,T,EPS
      EXTERNAL VA
* INTEGRATE EQUATIONS OF MOTION FROM X0 TO X1
* T=INTEGRAL ( DX / V(X) )
* VA(X,V,A) IS AN EXTERNAL ROUTINE THAT SUPLLIES V=DX/DT AND A=DV/DT
      REAL X, DX, V1, A1, V2, A2, D

      T=0
      X=X0
      CALL VA(X,V1,A1)
  10  CONTINUE
      DX=EPS*(X1-X0)
      X=X+DX
      IF(X.GT.X1)THEN
          DX=DX+X1-X
          X=X1
      ENDIF
 12   CALL VA(X,V2,A2)
      D=(V1+V2)**2-1.3333333*(A2-A1)*DX
      IF(D.LE.0)THEN
          DX=0.5*DX
          X =X-DX
          GOTO 12
      ENDIF
      T=T + 4.0*DX / ( V1+V2 + SQRT(D) )
      V1=V2
      A1=A2
      IF (X.LT.X1) GOTO 10
      END

      SUBROUTINE FPLTOT(R,U,d1,d2,POT,DPOT,M)
      implicit NONE
      REAL R,U,d1,d2,POT,DPOT
      INTEGER M
* SUMMATION OF POTENTIAL AND DERIVATIVE OVER A FEW PLANES
* INPUT QUANTITIES IN UNITS OF SCREENING RADIUS
* M=1: MOLIERE,  M=2: ZBL, M=4: Hartree-Fock
      REAL D,Y,DN,PP,DP,PP1,DP1
      INTEGER K
      D=d1+d2
      Y=R
      K=0
      POT=0
      DPOT=0
 10   DN=0.0
      Y=AMOD(Y,D)
 12   CALL FPL(Y+DN*D,DN*D,U,PP,DP,M)
      CALL FPL(Y-D-DN*D,-D-DN*D,U,PP1,DP1,M)
      PP=PP+PP1
      DP=DP+DP1
      POT=PP+POT
      DPOT=DP+DPOT
      IF (DN.LE.20.0)THEN
          DN=DN+1.0
          GOTO 12
      ENDIF
      IF (K.EQ.1) GOTO 30
      K=1
      IF (d2.EQ.0) GOTO 30
      Y=Y+d2
      GOTO 10
 30   END

      SUBROUTINE FPL(R,R0,U,PP,DP,M)
      implicit NONE
      REAL R, R0, U, PP, DP
      INTEGER M
* CONTRIBUTION TO POTENTIAL AND DERIVATIVE FROM ONE SINGLE PLANE.
* M=1: MOLIERE, M=2: ZBL, M=3: LINDHARD, M=4: Hartree-Fock
      REAL G,POTPL,DPOTPL
      EXTERNAL G,POTPL,DPOTPL
      REAL ALFA(4,2),BETA(4,2)
      REAL X,TMP,TMPP,TMDP,BR,BU,XU,G1,G2,F,X3,X03,X0,TMPP0
      INTEGER I,N
      DATA (ALFA(I,1),I=1,3)/0.2,1.1,0.7/
      DATA (BETA(I,1),I=1,3)/6.0,1.2,0.3/
      DATA (ALFA(I,2),I=1,4) /0.3636,1.0198,0.5604,0.05634/
      DATA (BETA(I,2),I=1,4)/3.2,0.9423,0.4029,0.2106/

      X=ABS(R)
      IF(M.EQ.3)GOTO 7
      IF(M.EQ.4)GOTO 8
      N=3
      IF (M.EQ.2) N=4
      TMPP=0
      TMDP=0
      DO 10 I=1,N
      BR=BETA(I,M)*X
* AVOID EXPONENT OF LARGE NEGATIVE NUMBER, TAKE 0 INSTEAD.
      IF(BR.GT.100.0)GOTO 10
* POT AND DERIVATIVE FOLDED WITH GAUSSIAN WITH STANDARD DEVIATION U:
      IF(U.EQ.0)GOTO 5
      XU=X/(U*SQRT(2.0))
      BU=BETA(I,M)*U/SQRT(2.0)
      IF(XU.GT.BU+4.0)THEN
* FAR AWAY FROM X=0 THERE IS STILL A CORRECTION EXP(BU*BU):
          TMP=ALFA(I,M)*EXP(BU*BU-BR)
          GOTO 6
      ENDIF
      G1=G(XU-BU)
      G2=G(-XU-BU)
      F=ALFA(I,M)*EXP(-XU*XU)
      TMPP= (G1+G2)*F/BETA(I,M) +TMPP
      TMDP= (G1-G2)*F           +TMDP
      GOTO 10
* NON-MODIFIED MOLIERE OR ZBL POTENTIAL:
  5   TMP=ALFA(I,M)*EXP(-BR)
  6   TMPP= TMP/BETA(I,M) +TMPP
      TMDP= TMP           +TMDP
 10   CONTINUE
      GOTO 20
* LINDHARD:
  7   X3=SQRT(X*X+3)
      TMPP=6.0/(X+X3)
      TMDP=TMPP/X3
      X0=ABS(R0)
      X03=SQRT(X0*X0+3.0)
      TMPP0=6.0/(X0+X03)
      TMPP=(X0-X)*(TMPP+TMPP0)/(X3+X03)
      GOTO 20
* Hartree-Fock
  8   TMPP=POTPL(X)
      TMDP=DPOTPL(X)
 20   IF(R.LT.0)TMDP=-TMDP
      PP=TMPP
      DP=TMDP
      END

      SUBROUTINE VA(X,V,A)
      implicit NONE
      REAL X,V,A
      REAL SQT
      EXTERNAL SQT
      REAL ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0
      COMMON /COM/ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0
      REAL POT,DPOT
      CALL FPLTOT(X/ATF,U1/ATF,d1/ATF,d2/ATF,POT,DPOT,1)
      V = SQT( VM0,FACM*ATF*POT,E0 )
      A=FACM*DPOT*0.5/E0
      END

      SUBROUTINE VAB(X,VB,AB)
      implicit NONE
      REAL X,VB,AB
      REAL SQT
      EXTERNAL SQT
      REAL ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0
      COMMON /COM/ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0
      REAL POT,DPOT
      CALL FPLTOT(X/ATB,U1/ATB,d1/ATB,d2/ATB,POT,DPOT,2)
      VB= SQT( VB0,FACB*ATB*POT,E0 )
      AB=FACB*DPOT*0.5/E0
      END

      SUBROUTINE VAL(X,VL,AL)
      implicit NONE
      REAL X,VL,AL
      REAL SQT
      EXTERNAL SQT
      REAL ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0
      COMMON /COM/ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0
      REAL POT,DPOT
      CALL FPLTOT(X/ATL,U1/ATL,d1/ATL,d2/ATL,POT,DPOT,3)
      VL=SQRT( (VL0-FACL*ATL*POT)/E0 )
      VL= SQT( VL0,FACL*ATL*POT,E0 )
      AL=FACL*DPOT*0.5/E0
      END

      SUBROUTINE VAH(X,VH,AH)
      implicit NONE
      REAL X,VH,AH
      REAL SQT
      EXTERNAL SQT
      REAL ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0
      COMMON /COM/ATF,ATB,ATL,U1,FACM,FACB,FACL,E0,XM,VM0,VB0,VL0,d1,d2
     + ,FACH, VH0
      REAL POT,DPOT
      CALL FPLTOT(X,U1,d1,d2,POT,DPOT,4)
      VH= SQT( VH0,FACH*POT,E0 )
      AH=FACH*DPOT*0.5/E0
      END

      FUNCTION SQT(A,B,C)
      implicit NONE
      REAL SQT,A,B,C,D
      D=(A-B)/C
      IF(D.GE.0)THEN
          SQT=SQRT(D)
      ELSE
          SQT=0.0
          if((B-A).gt. 1.0E-5*B) then 
            WRITE(6,1)A,B,B-A
            PAUSE
          endif
      ENDIF
    1 FORMAT('**SQT ERROR** A,B,A-B:',3E15.4)
      END

C ------------------------------------------------- FUNCTION G
      FUNCTION G(W)
      implicit NONE
      REAL G,W
C   EXP(W*W) * INTEGRAL FROM -INF TO W OF EXP(-X*X) / SQRT(PI)
      REAL X,Y,FY,FZ,WA,FW,Z
      FY(X) = 1.0 / ( 1.0 + 0.3275911*X )
      FZ(Y) = ((((0.94064607*Y-1.287822453) * Y + 1.25969513)
     2     * Y - 0.252128668) * Y + 0.225836846) * Y * 0.564189585
      WA = ABS(W)
          FW = FZ( FY(WA) )
      IF ( W .LE. 0.0 ) THEN
          Z = FW
      ELSE
          Z = EXP(WA*WA) - FW
      ENDIF
      G = Z
      END
C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FUNCTION ZZE2D(Z1,Z2,D)
      FUNCTION ZZE2D(Z1,Z2,D)
      implicit NONE
      real ZZE2D,Z1,Z2,D
C usage:  R = ZZE2D(Z1,Z2,D)
C returns the Coulomb energy (in MeV) of charges Z1 and Z2 at mutual
C distance D (Z1 and Z2 given in elementary charges, D in Angstrom)
C
C BOHR RADIUS IN ANGSTROM, RYDBERG CONSTANT IN MeV:
      REAL ABOHR, RYDBERG
      PARAMETER(ABOHR=0.5291770,RYDBERG=1.360580E-5)
      ZZE2D=Z1*Z2*2.0*ABOHR*RYDBERG/D
      END
C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ SUBROUTINE ask (ARG,PROMPT)
      SUBROUTINE ask(ARG,PROMPT)
      implicit NONE
C usage: CALL ask(ARG,PROMPT)
      REAL ARG
      CHARACTER*(*) PROMPT
  1   FORMAT(1X,A,A1,G10.3)
  2   FORMAT(1X,A,'[',G10.3,']:',$)
      WRITE(6,2)PROMPT,ARG
      READ (5,*)ARG
      WRITE(6,1) PROMPT,':',ARG
      END
C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FUNCTION AVXSQ(A,THETA,T)
      FUNCTION AVXSQ(A,THETA,T)
      implicit NONE
      REAL AVXSQ,A,THETA,T
C usage:  R = AVXSQ(A,THETA,T)
CALCULATE AVERAGE X**2 IN DEBYE MODEL.
C RESULT IN SQUARE ANGSTROMS.
C A IN ATOMIC MASS UNITS, THETA AND T IN KELVINS.
C EXTERNAL FUNCTION CALLED: QZ
      REAL QZ
      EXTERNAL QZ
C PARAMETERS IN S.I. UNITS:
      REAL HBAR,C,BOLS
      PARAMETER(HBAR=1.054589E-34,C=2.997925E8, BOLS=1.380662E-23)
C CONVERSION FACTORS, AMU TO MeV, METER TO ANGSTROM, JOULE TO MeV:
      REAL AMU,METER,JOULE
      PARAMETER(AMU=931.5016,METER=1.0E10,JOULE=1.0E-6/1.602189E-19)
CONSTANT APPEARING IN THE EXPRESSION FOR X**2:
      REAL CONS
      PARAMETER (CONS=0.75*(HBAR*C*METER)**2*JOULE/AMU/BOLS)
C
C NOTE:
C     if the input value of THETA is negative, it is considerd
C     to be the value of -x in Angstrom units.
C     In that case no calculations are done and A and T are ignored
C
      REAL Z,P
C
      IF ( THETA .LE. 0.0 ) THEN
          AVXSQ=THETA*THETA
          RETURN
      ENDIF
      Z=THETA/T
      P=1.0+4.0*QZ(Z)/(Z*Z)
      AVXSQ=CONS*P/(A*THETA)
      END
C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FUNCTION QZ(Z)
      FUNCTION QZ(Z)
      implicit NONE
      real QZ,Z
C usage:  R = QZ(Z)
C RETURNS QZ(Z)= INTEGRAL FROM 0 TO Z OF X/(EXP(X)-1)
C CALLED BY AVXSQ
      REAL PI,PI26
      PARAMETER(PI=3.14159265,PI26=PI**2/6.0)
      REAL Q,RJ,ZJ,T
      IF(Z .LT. 0.3)THEN
C POWER SERIES FOR SMALL Z:
          QZ=((Z/36.0 - 0.25) * Z + 1.0) * Z
      ELSE
C SERIES IN TERMS OF EXP(-J*Z) FOR LARGER Z:
          Q=PI26
          RJ=1.0
   1      ZJ=Z*RJ
          IF (ZJ .GT. 20.0) GOTO 2
          T=(ZJ + 1.0) / (EXP(ZJ) * RJ * RJ)
          Q=Q-T
          RJ=RJ+1.0
          IF (T .GT. 1.0E-8) GOTO 1
   2      QZ=Q
      ENDIF
      END
C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FUNCTION POTPL
      FUNCTION POTPL(X)
      implicit none
      real POTPL,X
CONTINUUM PLANAR POTENTIAL (Hartree-Fock)
C to obtain the planar potential in MeV the result should be
C multiplied by PI*ADENS*ZZE2D(Z1,Z2,1.0)
C where ADENS = areal density = atoms per square angstrom.

      include 'compotz.com'

      REAL DPOTPL
      EXTERNAL DPOTPL
      REAL POTP, PREC, YMAX, H
      INTEGER KERR
      YMAX= R(MAX(IZ2),IZ2)
      IF(X .GE. YMAX) THEN
        POTPL=0.0
        RETURN
      ENDIF
      PREC=YMAX*1.0E-6
      CALL ROMBERG(DPOTPL,X,YMAX,0.0001,POTP,PREC,H,KERR)
      IF(KERR.NE.0)WRITE(6,125)KERR,'DPOTPL'
  125 FORMAT(' ROMBERG: KERR=',I5,A10)
      POTPL=POTP
      END
C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FUNCTION DPOTPL
      FUNCTION DPOTPL(Z)
      implicit none
      real DPOTPL, Z
C MINUS DERIVATIVE OF CONTINUUM PLANAR POTENTIAL (Hartree-Fock)
C Z IN ANGSTROM UNITS
C The parameter IZ2 is passed via common block COMPOTZ

      include 'compotz.com'

      DPOTPL= APOLNOX(Z,R(1,IZ2),PH,4,MAX(IZ2))*2.0
      END
