C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ FUNCTION BOHR(Z1,Z2,ATPV,DEPTH)
      REAL FUNCTION BOHR(Z1,Z2,ATPV,DEPTH)
      IMPLICIT NONE
      REAL Z1,Z2,ATPV,DEPTH

C usage:  R = BOHR(Z1,Z2,ATPV,DEPTH)
C BOHR STRAGGLING
C    Z1,Z2  = CHARGE OF PROJECTILE AND TARGET, IN UNITS OF e.
C    DEPTH  = THE TARGET LAYER THICKNESS (in nm)
C    ATPV   = ATOMS PER cm3 * 1.0E22
C    BOHR   = VARIANCE OF STRAGGLING DISTRIBUTION (in MeV^2)
C
C     parameter c   : speed of light in m/s
C               e   : elementary charge in Coulomb
C               alfa: 1/(4*pi*epsilon0)
C               esq : e*e/(4*pi*epsilon0) in eV.m
      REAL c,e,alfa,esq,equad,fourpi,P10,fBohr
      parameter(c=2.99792458E8)
      parameter(e=1.6021892E-19)
      parameter(alfa=c*c*1.0E-7)
      parameter(esq=e*alfa)
      parameter(equad=esq*esq)
      parameter(fourpi=12.56637061)
      parameter(P10=1.0E7)
      parameter(fBohr=fourpi*equad*P10)
      BOHR =  Z1*Z1*Z2 * fBohr * ATPV *DEPTH
      END
