C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ SUBROUTINE ENORM(X,Y,SIG)
      SUBROUTINE ENORM ( X, Y, SIG )
C usage:  CALL ENORM ( X, Y, SIG )
C GET A PAIR OF RANDOM NUMBERS (X,Y) FROM A NORMAL DISTRIBUTION
C WITH STANDARD DEVIATION SIG.
C METHOD: IF RN1 AND RN2 ARE RANDOM SAMPLES FROM A UNIFORM
C  DISTRIBUTION IN [0..1], THEN
C   R = SIG * SQRT ( -2.0 * ALOG (RN1) )
C   X = R * COS ( TWOPI * RN2 )
C   Y = R * SIN ( TWOPI * RN2 )
C TO SPEED UP THINGS, TABLE LOOKUP IS USED FOR R IN THE RANGE
C 0.1 < RN1 < 0.9, AND FOR ALL VALUES OF RN2.
C
C USES SUBROUTINE RANMAR TO SUPPLY PSEUDORANDOM NUMBERS.
C MAKE SURE RMARIN (or RANSET) IS CALLED TO INITIALIZE RANMAR
C BEFORE THE FIRST CALL TO ENORM.
C
      REAL FR(100:900),FS(0:400),UNIRAN(2)
      LOGICAL INIT
      SAVE INIT,FR,FS
      DATA INIT/.FALSE./
      IF(INIT)GOTO 300
C COME HERE ONLY AT FIRST CALL. INITIALIZE TABLES OF R AND SIN
      DO 100 I=100,900
  100 FR(I)=SQRT(-2.0*ALOG(FLOAT(I)/1000.0))
      DO 200 I=0,400
  200 FS(I)=SIN(FLOAT(I)/400.0*1.5707963)
      INIT=.TRUE.
C COME HERE AT FIRST AND SUBSEQUENT CALLS
  300 CALL RANMAR(UNIRAN,2)
C
C CHOOSE A VALUE OF R
      RN=UNIRAN(1)
      IR=NINT(RN*1000.0)
      IF (IR.GE.100 .AND. IR.LE.900) THEN
          R=SIG*FR(IR)
      ELSE
          R=SIG*SQRT(-2.0*ALOG(RN))
      ENDIF
C
C CHOOSE AN ANGLE AND CALCULATE X AND Y
      IS=NINT(UNIRAN(2)*1600.0)
      IC=IS
      IF(IS.GT.800)THEN
          IS=IS-800
          IF(IS.GT.400)IS=800-IS
          X=-R*FS(IS)
      ELSE
          IF(IS.GT.400)IS=800-IS
          X= R*FS(IS)
      ENDIF
      IF(IC.GT.800)IC=1600-IC
      IF(IC.GT.400)THEN
          Y=-R*FS(IC-400)
      ELSE
          Y= R*FS(400-IC)
      ENDIF
      END
C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ SUBROUTINE PNORM(X,Y,SIG,UNIRAN)
      SUBROUTINE PNORM ( X, Y, SIG, UNIRAN )
      REAL X,Y,SIG,UNIRAN(2)
C a variation of ENORM where UNIRAN is supplied from outside      
C usage:  CALL PNORM ( X, Y, SIG , UNIRAN)
C GET A PAIR OF RANDOM NUMBERS (X,Y) FROM A NORMAL DISTRIBUTION
C WITH STANDARD DEVIATION SIG.
C METHOD: IF RN1 AND RN2 ARE RANDOM SAMPLES FROM A UNIFORM
C  DISTRIBUTION IN [0..1], THEN
C   R = SIG * SQRT ( -2.0 * ALOG (RN1) )
C   X = R * COS ( TWOPI * RN2 )
C   Y = R * SIN ( TWOPI * RN2 )
C TO SPEED UP THINGS, TABLE LOOKUP IS USED FOR R IN THE RANGE
C 0.1 < RN1 < 0.9, AND FOR ALL VALUES OF RN2.
C
      REAL FR(100:900),FS(0:400)
      LOGICAL INIT
      SAVE INIT,FR,FS
      DATA INIT/.FALSE./
      IF(INIT)GOTO 300
C COME HERE ONLY AT FIRST CALL. INITIALIZE TABLES OF R AND SIN
      DO 100 I=100,900
  100 FR(I)=SQRT(-2.0*ALOG(FLOAT(I)/1000.0))
      DO 200 I=0,400
  200 FS(I)=SIN(FLOAT(I)/400.0*1.5707963)
      INIT=.TRUE.
C COME HERE AT FIRST AND SUBSEQUENT CALLS
  300 CONTINUE
C
C CHOOSE A VALUE OF R
      RN=UNIRAN(1)
      IR=NINT(RN*1000.0)
      IF (IR.GE.100 .AND. IR.LE.900) THEN
          R=SIG*FR(IR)
      ELSE
          R=SIG*SQRT(-2.0*ALOG(RN))
      ENDIF
C
C CHOOSE AN ANGLE AND CALCULATE X AND Y
      IS=NINT(UNIRAN(2)*1600.0)
      IC=IS
      IF(IS.GT.800)THEN
          IS=IS-800
          IF(IS.GT.400)IS=800-IS
          X=-R*FS(IS)
      ELSE
          IF(IS.GT.400)IS=800-IS
          X= R*FS(IS)
      ENDIF
      IF(IC.GT.800)IC=1600-IC
      IF(IC.GT.400)THEN
          Y=-R*FS(IC-400)
      ELSE
          Y= R*FS(400-IC)
      ENDIF
      END
