C ~~~~~~~~~~~~~~~~~~~~~~~~~~ PORTABLE SUBROUTINES ~~~~~~~~~~~~~~~~~~~~~~
C
C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ MOLIERE - ZBL
C NOTE.  POTM AND DPOTM SHOULD ONLY BE USED FOR TEST PURPOSES.
C         A BETTER APPROXIMATION IS BUILT IN IN VTM AND DVTM.
C THE FOLLOWING PACKAGE OF ROUTINES DEALS WITH THE UNIVERSAL POTENTIAL.
C REF: ZIEGLER,BIERSACK,LITTMARK (1985)
C      "STOPPING POWERS AND RANGES", PERGAMON PRESS, CHAPTER 2.
C AS A RULE, DISTANCES ARE IN UNITS OF THE THOMAS-FERMI RADIUS,
C POTENTIAL AND FORCE IN UNITS OF Z1*Z2*E**2/D.
C SURVEY:
C FORCE   : CALCULATES FORCE IN X,Y FROM ATOM AT XJ,YJ
C VTM     : THERMALLY MODIFIED CONTINUUM STRING POTENTIAL
C DVTM    : MINUS FIRST DERIVATIVE (D/D(R/A)) OF VTM
C POT     : CONTINUUM STRING POTENTIAL
C DPOT    : MINUS FIRST DERIVATIVE OF POT
C POTM    : APPROXIMATION FOR R>>U OF THERMALLY MODIFIED CONTINUUM
C         STRING POTENTIAL.
C DPOTM   : MINUS FIRST DERIVATIVE OF POTM.
C ROMBERG : A GENERAL INTEGRATION ROUTINE
C BESK    : MODIFIED BESSEL FUNCTIONS ( K BESSEL FUNCTIONS)
C
      SUBROUTINE FORCE(FX,FY,X,Y,XJ,YJ)
C usage:  CALL FORCE(FX,FY,X,Y,XJ,YJ)
CALCULATES FORCE IN X,Y FROM ATOM AT XJ,YJ
      COMMON /COMPOTE/ ZZD,ZZA,ATF,U2A,U2ASQ,RHOA,XUNA,YUNA
      RX=(X-XJ)*XUNA
      RY=(Y-YJ)*YUNA
      RHOA=SQRT(RX**2+RY**2)
      F=ZZD*DVTM(RHOA)/ATF
      FX=F*RX/RHOA
      FY=F*RY/RHOA
      END
