C ------------------------------------------------ SUBROUTINE PRELIMI
      SUBROUTINE PRELIMI
      IMPLICIT NONE
C PRELIMINARY CALCULATIONS. CALLED BY MAIN PROGRAM FLUX .

      INCLUDE 'FLUX7.COM' 

      INTEGER I,J,KL,IL,LC,IND,IZ,NZMAX,LAY, IL1, IL2, L, M, ISTEP
      REAL ZZE2D, AVXSQ, U1SQ
      REAL ANGMAX, BIN, ZEMAX, DELZ, ZZ, GAUSS, DELCORE, CURPHI
      REAL CPH, SPH, CURDP, CDP, SDP
      REAL YUM1, XUM1
      REAL DUDMAX
      REAL SCREENING
      CHARACTER*20 TIME
      EXTERNAL ZZE2D, AVXSQ, SCREENING

C UD = DISTANCE BETWEEN ATOMS ALONG THE STRING
C RSCREEN(L) = SCREENING RADIUS IN ANGSTROM
C ZZA(L),ZZD(L) : Z1*Z2*E**2/ATF, Z1*Z2*E**2/UD , IN MeV
C XUN,YUN : DIMENSIONS OF UNIT CELL, IN ANGSTROM
C CELL : voume of unit cell in angstrom**3
C XATOM(I),YATOM(I) : STRING POSITION WITHIN UNIT CELL, IN ANGSTROM.
C U1(L) = STANDARD DEVIATION OF THERMALLY VIBRATING ATOM
C U2(L) = SAME * SQRT(2)
C RM(L) = MASS RATIO A1/A2
C ZSIG = STANDARD DEVIATION IMPURITY DISTRIBUTION
C TESQMAX: WHEN COS(TETA)**2 < TESQMAX A TRACK IS NO LONGER FOLLOWED
C ZZ0 = -ZIMAX: INITIAL VALUE OF Z.
C ZMAX : TRACKS ARE FOLLOWED FROM ZZ0 TO +ZMAX
C NDELZ: number of Z steps per bin of RAAK,DAV,DDAV
C MDELZ: number of Z periods per bin of RAAK,DAV,DDAV
C ANORM0 = NORMALISATION CONSTANT FOR CLOSE ENCOUNTER YIELD.
C DEVL = ELECTRONIC ENERGY LOSS PER UNIT OF Z (LOCAL VALENCE)
C DEVP =     "         "    "    "     "      (DISTANT VALENCE)
C ELCORE(I,L) = LN(DELTAE(B)), CORE ELECTRONS

      CLOSE(INLU)

C Open the coordinates file if specified
      IF(JOPTION(LFCFILE))THEN
        CALL LFOPEN(LUN(4),CFILE,'OLD',0,0)
      ENDIF

C DERIVE A RANDOM SEED FROM THE CLOCK, IF USER DID NOT SUPPLY ONE.
      IF(NSEED.EQ.0)THEN
        CALL SYSTIM(TIME)
        DO 440 I=19,1,-1
          J=ICHAR( TIME(I:I) )
          NSEED=(NSEED+J)*663608942+1
  440   CONTINUE
      ENDIF
C SEED THE RANDOM NUMBER GENERATOR
      CALL RANSET(NSEED)

      DO 67 KL=1,NLAY
      XUN(KL)= 0
      YUN(KL)= 0
      UD(KL) = 0
      if(NLZ(KL).eq.0)then
        DO 68 LC=1,NLC(KL)
        XUN(KL)= XUN(KL)+ UA(LC,KL)*XUNIT(LC,KL)
        YUN(KL)= YUN(KL)+ UA(LC,KL)*YUNIT(LC,KL)
        UD(KL) = UD(KL) + UA(LC,KL)*DUNIT(LC,KL) 
  68    CONTINUE
      else
        UD(KL)  =  DUNIT(1,KL)/MODZ(KL)
        XUN(KL) =  XUNIT(1,KL)
        YUN(KL) =  YUNIT(1,KL)
        DO 12 ISTEP=0,MODZ(KL)
C       Change UZ(ISTEP) from UD coordinates to real Z coordinates
        UZ(ISTEP,KL)=UZ(ISTEP,KL)*UD(KL)*MODZ(KL)
  12    CONTINUE
C       Calculate the different MODZ steps between the MODZ planes
        DO 1 ISTEP=0,MODZ(KL)-1
        DUZ(ISTEP,KL)=UZ(ISTEP+1,KL)-UZ(ISTEP,KL)
        WRITE(6,*) 'DUZ(',ISTEP,')=',DUZ(ISTEP,KL)
  1     CONTINUE
      endif
  
      IF(NLC(KL).eq.1)THEN
C cubic
        CELL(KL)= UA(1,KL)**3
      ELSE IF(NLC(KL).eq.2)THEN
        IF(CSYMM(KL).eq.'hexagonal')THEN
C hexagonal
          CELL(KL)= 0.8660254*UA(1,KL)*UA(1,KL)*UA(2,KL)
        ELSE IF(CSYMM(KL).eq.'tetragonal')THEN
C tetragonal
          CELL(KL)= UA(1,KL)*UA(1,KL)*UA(2,KL)
        ELSE
          stop 'unknown crystal class'
        ENDIF
      ELSE IF(NLC(KL).eq.3)THEN
        CELL(KL)=UA(1,KL)*UA(2,KL)*UA(3,KL)
      ELSE
        stop 'illegal number of lattice constants'
      ENDIF
      TWOXUN(KL) = 2.0 * XUN(KL)
      TWOYUN(KL) = 2.0 * YUN(KL)
      DO 2 IND=1,NIND(KL)
        XATOM(IND,KL) = XATOM0(IND,KL) * XUN(KL)
        YATOM(IND,KL) = YATOM0(IND,KL) * YUN(KL)
    2 CONTINUE

C PERIOD OF TRUE PRIMITIVE UNIT CELL
      XUM(KL)= MODX(KL)*XUN(KL)
      YUM(KL)= MODY(KL)*YUN(KL)
   67 CONTINUE

C TO MAKE SURE OF AN EVEN DISTRIBUTION OF INITIAL COORDINATES A
C MESH OF NX*NY SQUARES IS PUT OVER THE UNIT CELL. THE INITIAL
C COORDINATES ARE CHOSEN AT RANDOM, WITHIN EACH OF THESE SQUARES.
C THIS PROCEDURE MODIFIES THE VALUE OF NTRX SOMEWHAT.
      XUM1=XUM(1)
      YUM1=YUM(1)
      if(BEAMX .gt. 0.0)XUM1=BEAMX
      if(BEAMY .gt. 0.0)YUM1=BEAMY
      NXI=SQRT(NTRX*XUM1/YUM1)+0.5
      NYI=SQRT(NTRX*YUM1/XUM1)+0.5
      NTRX=NXI*NYI
      DXI =XUM1/NXI
      DYI =YUM1/NYI

C Map index of array ATOMFRAC(IL,KL) to index of ZPOS(NL,KL)
C Also, set atomfrac to 1 for the unmixed case
      DO 21 KL=1,NLAY
      IL=0
      DO 20 IL1=1,NL
      if(IL1.gt.NLA0(KL))then
        NLA1(IL1,KL)=NLA1(IL1-1,KL)
      endif
      if(.not. JOPTION(LMIXED)) then
        IF(IL1.le.NLA0(KL))IL=IL+1
        ILMAP(IL1,KL)= IL
        ATOMFRAC(IL,KL)= 1
      else
        IL=IL+1
        ILMAP(IL1,KL)= IL
        if(NLA1(IL1,KL).gt.1) IL=IL+NLA1(IL1,KL)-1
        IF(IL1.eq.NLA0(KL)) IL=0
      endif
   20 CONTINUE
   21 CONTINUE

C HOW TO SUBDIVIDE THE FLUX ARRAY, and the force field arrays FFX,FFY
      IF(NFA .LE. 0) NFA=NF
      NFX=NINT(SQRT(NFA*NFA*XUN(LFLX)/YUN(LFLX)))
      NFY=NFA*NFA/NFX
C
      DO 31 KL=1,NLAY
      IL=0
      DO 3 IL1=1,NLA0(KL)
      DO 3 IL2=1,NLA1(IL1,KL)
      IL=IL+1
C The following statement depends on the choice of ion-atom potential
      RSCREEN(IL,KL)=SCREENING(Z1, Z2(IL,KL),POTENTIAL)
      ZZA2(IL,KL)=ZZE2D(Z1,Z2(IL,KL),RSCREEN(IL,KL))
c      ZZD2(IL,KL)=ZZE2D(Z1,Z2(IL,KL),UD(KL)*ZPOS(IL1,KL))
c 27 feb 2008. replace the line above by
      if(NLZ(KL).eq.0)then
         ZZD2(IL,KL)=ZZE2D(Z1,Z2(IL,KL),UD(KL)*ZPOS(IL1,KL))
      else
         ZZD2(IL,KL)=ZZE2D(Z1,Z2(IL,KL),DUNIT(1,KL)*ZPOS(IL1,KL))
      endif

C if the 'Debye temperature' was given as a negative value, it is
C actually the thermal vibration amplitude in Angstrom units.
      IF(DEBYE(IL,KL) .LT. 0.0)THEN
        U1SQ=DEBYE(IL,KL)**2
      ELSE
        U1SQ=AVXSQ(A2(IL,KL),DEBYE(IL,KL),TEMP)
      ENDIF
      U1(IL,KL)= SQRT(U1SQ)
      U2SQ(IL,KL)= U1SQ * 2.0
      U2(IL,KL)= SQRT(U2SQ(IL,KL))
      RM(IL,KL)= A1/A2(IL,KL)
    3 CONTINUE
   31 CONTINUE
C
      ZSIG=ZIFWHM/SQRT(ALOG(256.0))
      if(NLZ(1).eq.0)then
        ANGMAX = MIN(XUN(1)/UD(1), YUN(1)/UD(1))
      else
        DUDMAX=DUZ(0,1)
        DO 4 ISTEP=1,MODZ(1)-1
        DUDMAX=MAX(DUDMAX,DUZ(ISTEP,1))
  4     CONTINUE
        ANGMAX = MIN ( XUN(1)/DUDMAX, YUN(1)/DUDMAX)
      endif
      if(TESQMAX.eq.0.0)TESQMAX = 1.0 / (ANGMAX*ANGMAX + 1.0)
CHOOSE ZMAX SUCH THAT:
C A. ZMAX - ZZ0 IS A MULTIPLE OF NBINA*UD
C B. ZMAX > ZIFWHM*1.5.
      ZZ0 = -ZIMAX
      BIN=NBINA
      ZEMAX=ZERO
      IF(ZIFWHM.GT.0)ZEMAX=ZIFWHM*1.5
      ZEMAX= ZEMAX-ZZ0
C NOW ZEMAX IS REQUIRED DEPTH.
C NDELZ nr of Z-STEPS per bin
C MDELZ nr of Z-PERIODS per bin
C DELZ IS BIN WIDTH IN ANGSTROM
      MDELZ = ZEMAX / (BIN*UD(1)*MODZ(1))
      IF ( MDELZ*BIN*UD(1) .LT. 0.99*ZEMAX ) MDELZ=MDELZ+1
      NDELZ = MDELZ*MODZ(1)
      DELZ = UD(1) * NDELZ
C NUMBER OF BINS TO BE USED ( .LE. NBINA): ZEMAX/DELZ ROUNDED UPWARDS
      NBINF = ZEMAX / DELZ
      IF ( NBINF*DELZ .LT. 0.99*ZEMAX ) NBINF=NBINF+1
      IF ( NBINF .GT. NBIN ) NBINF=NBIN
      ZMAX = ZZ0 + NBINF * DELZ
      NZMAX= NBINF * NDELZ
      write(6,*)'prelimi: bin-width:',DELZ, 'A, ',NDELZ,' zsteps,',
     + MDELZ,' z-periods'
      write(6,*)'prelimi: NBINF:',NBINF,' *DELZ=',NBINF * DELZ
C DETERMINE DEPTH PROFILE. ZZ AND ZSIG IN UNITS OF ANGSTROM
      DO 43 IZ=0,NBIN-1
      IF (IZ.LT.NBINF) THEN
C ZZ(IZ) IS AVERAGE Z COORDINATE OF IZ'TH BIN
        ZZ=ZZ0 + IZ*DELZ + (NDELZ-1)*0.5*UD(1)
        PROFIEL(IZ)=GAUSS(ZZ,ZSIG) * DELZ
      ELSE
        PROFIEL(IZ)=0
      ENDIF
  43  CONTINUE

      DO 422 KL=1,NLAY
      IL=0
      DO 421 IL1=1,NLA0(KL)
      DO 420 IL2=1,NLA1(IL1,KL)
      IL=IL+1
      APC1(IL,KL)=APC(IL1,KL)
      DO 41 L=1+NLA0(KL),NLS(KL)
      APC1(IL,KL)=APC1(IL,KL)+APC(L,KL)
  41  CONTINUE
  420 CONTINUE
  421 CONTINUE
  422 CONTINUE

      DO 451 KL=1,NLAY
      IL=0
      DO 450 IL1=1,NLA0(KL)
      DO 449 IL2=1,NLA1(IL1,KL)
      IL=IL+1
      if(APC1(IL,KL).ne.0.0)THEN
        ANORM0(IL,KL)=CELL(KL)/(PI*U2SQ(IL,KL)*NTRX*APC1(IL,KL)*DELZ)
      else
        ANORM0(IL,KL)= 0.0
      endif
      if (ATOMFRAC(IL,KL).ne.0) then
        ANORM0(IL,KL)= ANORM0(IL,KL)/ATOMFRAC(IL,KL)
      endif
 449  CONTINUE
 450  CONTINUE
 451  CONTINUE

      XSDR = XSD * DEGREE
      YSDR = YSD * DEGREE
      ANGR = ANGH* DEGREE
C
      IF(JOPTION(LFEFILE))THEN
C read the initial energy spectrum
        call INPEFILE
C THESE CHANGES WERE DONE AS THE DE/DX
C IS CALCULATED AT THE BEGINNING OF EACH TRACK IN THIS VERSION.
        DO 452 KL=1,NLAY
        TAPC(KL)=0.0
        AVZVAL(KL)=0.0
        IL=0
        DO 45 IL1=1,NLS(KL)
        DO 45 IL2=1,NLA1(IL1,KL)
        if(IL1.le.NLA0(KL))IL=IL+1
        TAPC(KL)=TAPC(KL)+APC(IL1,KL)
        AVZVAL(KL)=AVZVAL(KL)+ZVAL(IL,KL)*ATOMFRAC(IL,KL)*APC(IL1,KL)
   45   CONTINUE
        AVZVAL(KL)=AVZVAL(KL)/TAPC(KL)
C       call SDEDX to calculate DEDXVL,DEDXVP at average T0
C       T0 was already calculated in INPEFILE
C       These values of T0, DEDXVL, DEDXVP will not be used in BBLOOP
C       but only serve  as estimates of the average values.
        CALL SDEDX(DEDXVL(KL,1),DEDXVP(KL,1),T0,Z1,A1,
     +   AVZVAL(KL),CELL(KL)/TAPC(KL))
  452   CONTINUE
      ENDIF

      DO 81 M=1,NSTOP
      DO 80 KL=1,NLAY
      DEVL(KL,M) = DEDXVL(KL,M) * 1.0E-6 * UD(KL)
      DEVP(KL,M) = DEDXVP(KL,M) * 1.0E-6 * UD(KL)
   80 CONTINUE
   81 CONTINUE
      DO 53 M=1,NSTOP
      DO 52 KL=1,NLAY
      DO 51 IL=1,NLA(KL)
      DO 50 I=1,50
C PREVENT UNDERFLOW BY PUTTING LOWER LIMIT ON CORE ELECTRON E-LOSS:
      DELCORE=ELCORE(I,IL,KL,M)
      IF(DELCORE .EQ. 0)DELCORE = 1.0E-10
      ELCORE(I,IL,KL,M)=LOG ( DELCORE * 1.0E-6 )
   50 CONTINUE
   51 CONTINUE
   52 CONTINUE
   53 CONTINUE
C
      EBIN = (TFMAX - TFMIN)/NEBINA

C parameters for velocity distribution VFINAL
C Modified Aug. 1997.
C To get range symmetric we should take 1+NVA/2, not NVA/2.
C angular range corresponds to [ -ANGPMAX/2 .. +ANGPMAX/2 ]
C or more precise: range of (PX-P0) is -/+ SIN(ANGPMAX/2)
      ANGPMAX = ANGPMDG*DEGREE
      PBIN = 1
      IF(NVA.NE.0)PBIN = 2.0 * SIN(0.5*ANGPMAX) / NVA
      NP0 = 1 + NVA/2
C
      CRITX= CRITXDG*DEGREE
      CRITY= CRITYDG*DEGREE

curved crystal?
calculate the rotation of the impulse vector at every step,  CURMAT
calculate the rotation of the impulse vector over the layer, CURMATT
      DO 70 LAY=1,NLAY
      DO 71 I=1,3
      DO 72 J=1,3
      CURMAT(J,I,LAY)=0.0
      CURMATT(J,I,LAY)=0.0
  72  CONTINUE
      CURMAT(I,I,LAY)=1.0
      CURMATT(I,I,LAY)=1.0
  71  CONTINUE
      IF(CURRADIUS(LAY).NE.0)THEN
      CURPHI= (CURPLANE - 90.0)*PI/180.0
      CPH = COS(CURPHI)
      SPH = SIN(CURPHI)
      CURDP= 2.0 * ASIN(0.5*UD(LAY)/CURRADIUS(LAY))
      CDP = COS(CURDP)
      SDP = SIN(CURDP)
      CALL ROT(CURMAT(1,1,LAY),3,SPH,CPH)
      CALL ROT(CURMAT(1,1,LAY),1,SDP,CDP)
      CALL ROT(CURMAT(1,1,LAY),3,-SPH,CPH)
      CURDP= 2.0 * ASIN(0.5*ZINTERF(LAY)/CURRADIUS(LAY))
      CDP = COS(CURDP)
      SDP = SIN(CURDP)
      CALL ROT(CURMATT(1,1,LAY),3,SPH,CPH)
      CALL ROT(CURMATT(1,1,LAY),1,SDP,CDP)
      CALL ROT(CURMATT(1,1,LAY),3,-SPH,CPH)
      ENDIF
   70 CONTINUE
      CMAWBEAM=COS(AMAWBEAM*DEGREE)
      END
      SUBROUTINE ROT(R,I,S,C)
      IMPLICIT NONE
      REAL R(3,3)
      INTEGER I
      REAL S,C
C ROTATE MATRIX R ABOUT AXIS I, ANGLE WITH SINE S, AND COSINE C.

      REAL U(3,3),T(3,3)
      INTEGER J,K,L,M,N

      J=I+1
      IF(J.GT.3)J=1
      K=J+1
      IF(K.GT.3)K=1
      DO 1 L=1,3
      DO 1 M=1,3
  1   U(L,M)=0.0
      U(I,I)=1.0
      U(J,J)=C
      U(J,K)=S
      U(K,J)=-S
      U(K,K)=C
      DO 2 L=1,3
      DO 2 M=1,3
      T(L,M)=0.0
      DO 2 N=1,3
  2   T(L,M)=T(L,M)+U(L,N)*R(N,M)
      DO 3 L=1,3
      DO 3 M=1,3
  3   R(L,M)=T(L,M)
      END
