      SUBROUTINE scoef(z1,mm1,m1,m2,rho,atrho,vfermi,lfctr,pcoef,
     1 coef_flag)

c--------------------------------------------------------------------
c Input
c  z1     =  atomic number
c Output
c  mm1    =  atomic weight of most abundant isotope(rounded to
c            nearest integer)
c  m1     =  atomic weight of most abundant isotope
c  m2     =  natural atomic weight
c  rho    =  density (g/cm3)
c  atrho  =  atomic density (per 10E22 atoms/cm3)
c  vfermi =  fermi velocity
c  lfctr  =  screening factor
c  pcoef  =  proton stopping power coefficients
c-----------------------------------------------------------------------
c     implicit none

      integer      z1
      integer      z,i
      real      mm1,m1,m2,rho,atrho,vfermi,lfctr,sublim
      real      pcoef(8)
      integer coef_flag
      character*80 fullname
      external fullname

      if(z1.lt.1.0 .or. z1.gt.92.0) then
        write(6,'(A, F14.0)')'scoef: bad Z1',z1
        stop
      endif

      if(coef_flag.eq.0.or.coef_flag.eq.88) then
        open(unit=14,file=fullname('scoef.88'),status='old')
      else
        open(unit=14,file=fullname('scoef.84'),status='old')
      endif
      rewind(14)
      do 100 i=1,z1+1
        read(14,*)
100   continue
      read(14,*) z,mm1,m1,m2,rho,atrho,vfermi,lfctr,sublim
      do 123 i=1,92
        read(14,*)
123   continue
      read(14,*) z,pcoef
      close(14)
      atrho=atrho*1e22
      END
*********************************************************************
****  "rstop" : ion stopping cross-section subroutine used by trim  *
****  version :  1987     (c) 1985, j.f.ziegler, yorktown,ny,10598  *
****  adapted for program DEPTH, PJMS, 1991
*********************************************************************
*********     (output is 100 values of se (ev-ang.2))      ********
*********     nuclear stopping is added to electronic stopping  ****
********************************************************************
*********    Rz1 = ion atomic number           REAL         ********
*********    mm1= ion atomic mass              REAL         ********
*********    m1 = ion atomic weight (amu)      REAL         ********
*********    Rz2 = target atomic number        REAL         ********
*********    m2 = target atomic weight (amu)   REAL         ********
*********    ee = ion energy (kev)             REAL         ********
*********    rho = target density (g/cm3)      REAL         ********
*********    atrho=target density (atoms/cm3)  REAL         ********
*********    vfermi = (fermi velocity of solid) / v0        ********
*********    lfctr = lambda screening factor do ions        ********
*********    pcoef = stopping coefficients do protons       ********
*********    se = calculated electronic stopping (ev-a2)    ********
*********    enumb = number of energies         INTEGER !!  ********
*********    eflag = a flag                     INTEGER !!  ********
*********            if eflag = 0, then return 100          ********
*********            stopping values up to energy ee.       ********
*********            if eflag=1, then use enumb energies    ********
*********           hidden in se.                           ********
********************************************************************
********************************************************************
*********    need data file : scoef data (184,80)           ********
*********    scoef(*,1) = atomic number of element          ********
*********    scoef(*,2) = atomic mass of most abundant isotope *****
*********    scoef(*,3) = atomic weight of m.a.i.           ********
*********    scoef(*,4) = target mass (amu)                 ********
*********    scoef(*,5) = g/cm3 density (normal abundance)  ********
*********    scoef(*,6) = atoms/cm3*1e22 (normal abundance) ********
*********    scoef(*,7) = (fermi velocity of solid) / v0    ********
*********    scoef(*,8) = lambda screening factor do ions   ********
*********    scoef(92,*) to scoef(184,*) = coef.of proton stopping *
********************************************************************
********************************************************************
      SUBROUTINE rstop (Rz1,m1,Rz2,ee,se,enumb,eflag,coef_flag,atpv,m2)
      implicit real(a-z)
      integer z1, z2, i
      real pcoef(8)
      real ekev(129)
      real se(129)
      integer enumb, eflag, coef_flag
c     enumb and eflag changed from REAL to INTEGER type (PJMS 1991)
      character*80 fullname
      external fullname

* begin
      z1=nint(Rz1)
      z2=nint(Rz2)
* get lfctr; the screening length for the ion
      call scoef(z1,y1,mm1,y2,y3,y4   ,y5    ,lfctr,pcoef,coef_flag)
*   get vfermi for the target
      call scoef(z2,y1,y2 ,m2,y4,atrho,vfermi,y5   ,pcoef,coef_flag)

      if(m1.eq.0) m1=mm1
c     mm1 = mass (amu) of most abundant isotope.
      if(eflag.eq.0) then
c     get 100 se
        enumb=100 
c       100 equally spaced energies
        emax=ee/m1 
c       maximum energy of array
        e=ee/100/m1
        do 951 i=1,100
 951      ekev(i)=e*i 
c         array of energies
      else 
c     eflag=1 ! enumb values only
        emax=0
        do 952 i=1,enumb 
c       use preset energies (kev)
          ekev(i)=se(i)/m1
c         energies passed in array se.
          emax = amax1(emax,ekev(i)) 
c         maximum energy of array.
 952    continue
      endif
c
      if(emax.ge.10000) then 
c     get high energy stopping coefficients
        open(unit=14,file=fullname('hicoef.dat'),status='old')
        do 953 i=1,z2-1
          read(14,*)
 953    continue
        read(14,*) hi1,hi2,hi3,hi4
        close(14)
      endif

      do 954 i=1,enumb
        e=ekev(i)
*-- branch for electronic stopping of : z1=1, z1=2 or z1.gt.2 .
        if(z1.eq.1) then 
c       proton stopping
          call rpstop(z2,e,pcoef,se(i),hi1,hi2,hi3,hi4)
        else if(z1.eq.2) then 
c       he stopping powers
* velocity proportional stopping below kev/amu  >> he0 <<.
          he0=1.0
          he=amax1(he0,e)
          b=alog(he)
      a=.2865+.1266*b-.001429*b*b+.02402*b*b*b-.01135*b**4+.001475*b**5
          heh=1.0-exp(-amin1(30.,a))
***** add z1**3 effect to he/h stopping power ratio** heh **.
          if(he.lt.1) he=1 
c          added since 84
          a=(1.+(.007+.00005*Rz2)*exp(-(7.6-alog(he))**2)) 
          heh=heh*a*a
c          end addition 84
          call rpstop(z2,he,pcoef,sp,hi1,hi2,hi3,hi4)
          se(i)=sp*heh*4.
          if(e.le.he0) then
**** calc. he velocity proportional stopping
            se(i)=se(i)*sqrt(e/he0)
          endif
        else 
c       z greater than 2 so use heavy-ion stopping
* use velocity stopping for ((yrmin=vr/Rz1**.67).le.0.13)
* of for (vr.le.1.0)
          yrmin=0.13
          vrmin=1.0
          v=sqrt(e/25)/vfermi
          if(v.lt.1) then
            vr=(3*vfermi/4)*(1+(2*v*v/3)-v**4/15)
          else
            vr=v*vfermi*(1+1/(5*v*v))
          endif
* set yr = maximum of (vr/z1**.67),(vrmin/z1**.67) or yrmin.
          yr=amax1(yrmin,vr/Rz1**.6667)
          yr=amax1(yr,vrmin/Rz1**.6667)
          a=-.803*yr**0.3+1.3167*yr**0.6+.38157*yr+.008983*yr*yr
* q = ionization level of the ion at velocity * yr *.
          q=amin1(1.,amax1(0.,1.-exp(-amin1(a,50.))))
*** now we convert ionization level to effective charge.
          ll=(amin1(0.43,amax1(.32,.12+.025*Rz1)))/Rz1**.3333
*-- the following calculates the screening length of the ion, using
*-- brandt & kitagawa eq.(15) with constant a = .343.
*-- it includes s-shell correction shown in zbl, figure (3-24).
          l0=(.8-q*(amin1(1.2,0.6+Rz1/30.)))/Rz1**.3333
          if(q.lt.0.2) then
            l1=0.
          else if(q.lt.(amax1(0.,.9-.025*Rz1))) then
            q1=0.2
            l1=ll*(q-.2)/abs(amax1(0.,.9-.025*Rz1)-.2000001)
          else if((q.lt.(amax1(0.,1.0-.025*amin1(16.,1.0*Rz1))))) then
            l1=ll
            else
            l1=ll*(1.0-q)/(.025*amin1(16.,1.0*Rz1))
          endif
          l=amax1(l1,l0*lfctr)
      zeta=q+(1./(2.*vfermi**2))*(1.-q)*alog(1+(4*l*vfermi/1.919)**2)
*;!!!!!!    !**** add z1**3 effect as shown in ref. 779.    ! deleted si
*;!!!    a=-(7.6-amax1(0.,alog(e)))**2          ! deleted since 84
*;!!!    zeta=zeta*(1.+(1./z1**2)*(.18+.0015*z2)*exp(a)) ! deleted si
          if((yr.gt.amax1(yrmin,vrmin/Rz1**.6667))) then
            call rpstop(z2,e,pcoef,sp,hi1,hi2,hi3,hi4)
            se(i)=sp*(zeta*Rz1)**2
          else 
c         calculate velocity stopping for yr less than yrmin.
            vrmin=amax1(vrmin,yrmin*Rz1**.6667)
            vmin=.5*(vrmin+sqrt(amax1(0.,vrmin**2-0.8*vfermi**2)))
            eee=25*vmin**2
            call rpstop(z2,eee,pcoef,sp,hi1,hi2,hi3,hi4)
            power=0.5
*-- following compensates for band-gap of semiconductors
            if((z2.eq.14.or.z2.eq.32).and.(z1.le.19)) power=.375
            if(z2.eq.6) power=.375
            se(i)=(sp*(zeta*z1)**2)*(e/eee)**power 
c           power=0.5 in trim84
          endif
        endif
* nuclear stopping
       ee=ekev(i)*m1
       epsil = 32.53*m2*ee/(Rz1*Rz2*(m1+m2)*(Rz1**.23+Rz2**.23))
       if(epsil.lt.30.0) then
         a=(.01321*epsil**.21226)+(.19593*epsil**.5)
         sn=.5*alog(1+1.1383*epsil)/(epsil+a)
       else
         sn = alog(epsil)/(2*epsil)
       endif
       sn=sn*Rz1*Rz2*m1*8.462/((m1+m2)*(Rz1**.23+Rz2**.23))
         se(i)=se(i)+sn
 954   continue
c
      atpv=atrho
c
c convert to ev/A
*     do 956 i=1,enumb
*      se(i)=se(i)*atrho*1e-23
*956  continue
      END
*-------------------------------------------------------------------
* proton electronic stopping powers
* high energy stopping coefficients were added since trim84
*-------------------------------------------------------------------
      SUBROUTINE rpstop(z2,e,pcoef,sp,hi1,hi2,hi3,hi4)
      implicit real(a-z)
      integer z2
      real pcoef(8)
      if((e.gt.1.0E4)) then 
c     high energy stopping
        x=alog(e)/e      
c       ********* high energy stopping (6/87)
        sp=hi1+(hi2*x)+(hi3*x*x)+(hi4/x)
      else 
c     velocity proportional stopping below velocity ** pe0 **
        pe0=25.
        pe=amax1(pe0,e)
        sl=(pcoef(1)*pe**pcoef(2))+pcoef(3)*pe**pcoef(4)
        sh=pcoef(5)/pe**pcoef(6)*alog((pcoef(7)/pe)+pcoef(8)*pe)
        sp=sl*sh/(sl+sh)
        if((e.le.pe0)) then
***** velpwr is the power of velocity stopping below pe0.
          velpwr=0.45
          if(z2.le.6) velpwr=0.25
          sp=sp*(e/pe0)**velpwr
        endif
      endif
      END
