
C ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  SUBROUTINE SDEDX
C
      SUBROUTINE SDEDX(DEDXVL,DEDXVP,T0,Z1,A1,ZVAL,VOLAT)
CALCULATION OF THE ENERGY LOSS DUE TO ELECTRONS OF FAST IONS IN
CRYSTALS. THE RESULT IS CALCULATED IN UNITS eV/ANGSTROM!!
C THE INPUT PARAMETERS ARE 
C T0  = KINETIC ENERGY OF PARTICLE IN MeV
C Z1  = NUCLEAR CHARGE OF PARTICLE
C A1  = MASS OF PARTICLE IN AMU
C ZVAL= THE AVERAGE NUMBER OF VALENCE (I.E. DELOCALIZED) ELECTRONS 
C       PER ATOM IN THE THREE-DIMENSIONAL UNIT CELL.
C VOLAT = atomic volume in angstrom**3 
C IT IS NOT DISTINGUISHED BETWEEN ZLOCAL AND ZVALENCE, 
C IN CONTRAST TO SUBROUTINE DEDX.
C 

      PARAMETER(PI=3.14159265,TWOPI=2.0*PI,DEGREE=TWOPI/360.0)
C BOHR RADIUS IN ANGSTROM, RYDBERG CONSTANT IN MeV
      PARAMETER(ABOHR=0.5291770,RYDBERG=1.360580E-5)
C H,C,AND BOLTZMANN CONSTANT IN S.I. UNITS
      PARAMETER(HBAR=1.054589E-34,C=2.997925E8, BOLS=1.380662E-23)
C AMU IN MeV,METER IN ANGSTROMS, JOULE IN MeV
      REAL METER,JOULE
      PARAMETER(AMU=931.5016,METER=1.0E10,JOULE=1.0E-6/1.602189E-19)
      PARAMETER(AME=0.511003)
      PARAMETER (HC=HBAR*C*METER*JOULE)

      ZLOC=ZVAL
CALCULATE V/C OF ION:
      VION = SQRT( T0 * (T0+2.0*A1*AMU) ) / (T0+A1*AMU)
CALCULATE EFFECTIVE CHARGE=Z1(1-EXP(-(V/C*137/Z1**(2/3))))
      ZION = Z1 * (1.0 - EXP(-VION*137.0/Z1**(2.0/3.0)))
CALCULATE FACTOR 4*PI*(ZION*E**2)**2:
      ZFACT=4.0*PI*ZZE2D(ZION,1.0,1.0)**2
C ZFACT DIVIDED BY ELECTRON MASS * VION**2  * ATOMIC VOLUME:
      TE = AME * VION**2
      ZFACT = ZFACT / ( TE * VOLAT )
C ZFACT IS NOW THE 'BASIC UNIT' WITH DIMENSION MeV/ANGSTROM.
CALCULATE HBAR*OMEGA PLASMA IN MeV:
      HOMEGA = HC * SQRT(4.0*PI*ZZE2D(ZVAL,1.0,1.0)/(AME*VOLAT))
CALCULATE VFERMI/C
      VFERMI = HC * (3.0*PI*PI*ZVAL/VOLAT)**(1.0/3.0) / AME
CALCULATE DE/DX (VALENCE,LOCAL)
C 1. USING THE FORMULA OF PINES (= MELVIN AND TOMBRELLO)
      DEDXVP1= 1.0E6*ZFACT*ZLOC * ALOG( 2.0*AME*VION*VFERMI/ HOMEGA )

C2. USING THE FORMULA OF BELOSHITSKY AND KUMAKHOV
C      XM = HOMEGA/(AME*VION*VFERMI)
C      CALL BESK(XM,0,BK0,KER)
C      CALL BESK(XM,1,BK1,KER)
C      DEDXVP2=1.0E6*ZFACT*ZLOC*(XM*BK0*BK1-0.5*XM**2*(BK1-BK0)
C     +      *(BK1+BK0))
C 3. USING THE APPROXIMATION TO THE ABOVE GIVEN BY BELOSHITSKY & KUMAKHO
C      DEDXVP3=DEDXVP1 - 1.0E6*ZFACT*ZLOC*(0.577+0.5)

CALCULATE DE/DX (VALENCE):
      DEDXVAL= 1.0E6*ZFACT * ZVAL * ALOG(VION/VFERMI)

      DEDXVL=DEDXVAL
      DEDXVP=DEDXVP1


C PRINT SOME OF THE RESULTS.
C  444 FORMAT(2(1X,A10,': ',G13.5,1X,A6))
C  555 FORMAT(1X,' DE/DX ',A40,': ',G13.5,' eV/angstrom')
C      WRITE(*,444)'Z ION',Z1,'e','Z EFF',ZION,'e'
C      WRITE(*,444)'A ION',A1,'amu','ENERGY',T0,'MeV'
C      WRITE(*,444)'AT.VOL.',VOLAT ,'A**3','FACTOR',ZFACT,'MeV/A'
C      WRITE(*,444)'V ION',VION,'c','V FERMI',VFERMI,'c'
C      WRITE(*,444)'m*VION**2',TE,'MeV','h*OMEGA',HOMEGA,'MeV'
C      WRITE(*,555) '(VALENCE,COLLIS)                          ',
C     +    DEDXVAL
C      WRITE(*,555) '(VAL,PLASM) [PINES / MELVIN & TOMBRELLO]  ',
C     +    DEDXVP1
C      WRITE(*,555) '(VAL,PLASM) [BELOSHITSKY & KUMAKHOV] exact',
C     +   DEDXVP2
C      WRITE(*,555) '(VAL,PLASM) [BELOSHITSKY & KUMAKHOV] appr.',
C     +   DEDXVP3
C
      RETURN
      END
