C ~~~~~~~~~~~~~~~~~~~~~~~~~~~ SYMMETRIC AND UPPER TRIANGULAR MATRICES
C Subroutines for symmetric matrices S,
C and upper triangular matrices U.
C Of both types of matrices only the elements [i,j] with i .le. j
C are stored, in a one-dimensional array, such that [i,j] corresponds
C with array element (i+(j-1)*j/2). Explicitly the order is
C [1,1] [1,2] [2,2] [1,3] [2,3] [3,3] ...
C SYMDET factorizes a symmetric positive definite matrix S,
C        such that U(transpose).U = S. (Cholesky factorization)
C INPRODL calculates the inner products U(transpose).b
C UPINV  inverts an upper triangular matrix U
C
      SUBROUTINE SYMDET(A,X,N,DET)
C usage:  CALL SYMDET(A,X,N,DET)
      DOUBLE PRECISION R,C
      DIMENSION A(*),X(N)
C   DIMENSION OF ARRAY A IS N*(N+1)/2
C
C   CHOLESKY FACTORIZATION AND DETERMINANT
C   A IS REPLACED BY AN UPPER TRIANGULAR MATRIX U
C   SUCH THAT U(TRANSPOSE).U = A
C   X IS A WORK ARRAY OF LENGTH N
C   IF UNSUCCESFUL, RETURN WITH DET .LE. 0
      DET=1.0
      K0=0
      DO 150 K=1,N
      DO 110 I=1,K
110   X(I)=A(I+K0)
C Now X(i) is A[i,K]
      R=X(K)
      DO 120 J=1,K-1
      C=X(J)
120   R=R-C*C
C Now R is A[K,K] - sum( j<K, U[j,K]**2 )
      DET=DET*R
C   IF R.LE.0 MATRIX IS NOT POSITIVE DEFINITE
      IF(R.LE.0)RETURN
      R=SQRT(R)
      K0=K0+K
      A(K0)=R
C A[K,K] replaced by U[K,K]= sqrt(R)
      J0=K0
      DO 140 J=K+1,N
      C=A(K+J0)
      DO 130 I=1,K-1
130   C=C-A(I+J0)*X(I)
      A(K+J0)=C/R
C A[K,j] replaced by U[K,j]=(A[K,j]-sum(i<K,U[i,j]*U[i,K]))/U[K,K]
140   J0=J0+J
150   CONTINUE
      END
