      FUNCTION VTM(RHOA1)
C usage:  R = VTM(RHOA1)
C THIS FUNCTION SUBROUTINE CALCULATES THE THERMALLY MODIFIED
C CONTINUUM STRING POTENTIAL BY A DOUBLE INTEGRATION OVER THETA
C AND R OF:
C EXP(-R**2/U2**2)*U(SQRT(R**2+RHO**2-2*R*RHO*COS(THETA))/ATF)*2*R/U2**2
C WHERE U(R/ATF) IS THE STATIC CONTINUUM STRING POTENTIAL
C NORMALIZED: Z1*Z2*E**2/D == 1.
C
C USES U2A AND U2ASQ OF COMMON BLOCK COMPOTE
C SETS RHOA          "     "      "       "
C
      EXTERNAL GRANDM
      COMMON /COMPOTE/ ZZD,ZZA,ATF,U2A,U2ASQ,RHOA,XUNA,YUNA
      RHOA=RHOA1
      IF(RHOA.GT.U2A*5.0)THEN
          VTM= 0.5 * POT(RHOA) + 0.25 *
     *    ((1.0-0.5*U2A/RHOA)*POT(RHOA-U2A)+
     +     (1.0+0.5*U2A/RHOA)*POT(RHOA+U2A))
      ELSE
          XSIMAX=5.0*U2A+RHOA
          IF(XSIMAX.GT.75.0)XSIMAX=75.0
          XSI0=RHOA-6.0*U2A
          IF(XSI0.LT.1.0E-5)XSI0=1.0E-5
          SUMM=0.0
          XSI1=XSI0+0.125*U2A
    1     PREC=0.01*(XSI1-XSI0)
          CALL ROMBERG(GRANDM,XSI0,XSI1,1.0E-4,SINT,PREC,H,KERR)
          IF(KERR.NE.0)CALL ERRMESS('VTM',KERR)
          SUMM=SUMM+SINT
          IF(XSI1.LT.XSIMAX*0.999)THEN
              XSI0=XSI1
              XSI1=XSI1+0.125*U2A
              IF(XSI1.GT.XSIMAX)XSI1=XSIMAX
              GOTO 1
          ENDIF
          VTM=SUMM * 2.0 / U2ASQ
      ENDIF
      END
C
      FUNCTION GRANDM(X)
C usage:  R = GRANDM(X)
C THIS FUNCTION SUBROUTINE SUPPLIES THE INTEGRAND OF VTM.
      COMMON /COMPOTE/ ZZD,ZZA,ATF,U2A,U2ASQ,RHOA,XUNA,YUNA
      GRANDM=X*POT(X)/EXP((X-RHOA)**2/U2ASQ)*FI0(X*RHOA/U2ASQ)
      END
C
      FUNCTION FI0(A)
C usage:  R = FI0(A)
C FI0=EXP(-2A)*EXP(2ACOS(PHI)) AVERAGED OVER PHI.
      PARAMETER(PI=3.14159265)
      IF(A.GT.48.0)THEN
          FI0=1.0/SQRT(4.0*PI*A)*(1.0+0.0625/A)
      ELSE
          SUM=0.0
          TERM=EXP(-A)
          RN=0.0
  1       SUM1=SUM+TERM*TERM
          IF(SUM1.GT.SUM)THEN
              SUM=SUM1
              RN=RN+1.0
              TERM=TERM*A/RN
              GOTO 1
          ENDIF
          FI0=SUM1
      ENDIF
      END
