      program fxyfilter
      implicit NONE

c This program takes as input pairs of data (x, y)
c From these it builds up a frequency distribution f(x,y)
c in an NX x NY array, and outputs this array.
c The bin widths and the x,y values for each bin are further
c specified by XMIN, XMAX, YMIN, YMAX, see subroutine buildxy.

c The values for NX, NY, XMIN, XMAX, YMIN, YMAX
c are specified, either as commandline arguments, or, by a
c 6-line file of the name 'fxy.sel'. 

      character*70 args(6)
      real work(10)
      integer iwork(10), nargs, NX, NY
      real XMIN, XMAX, YMIN, YMAX
      integer i,ipt,iterm,ier
      integer lfopen
      external lfopen

      call f77start(nargs, args,6)
      if(nargs .lt. 6) then
        ier= lfopen(1,'fxy.sel','OLD',0,0)
        read(1,'(A70)')args(1)
        read(1,'(A70)')args(2)
        read(1,'(A70)')args(3)
        read(1,'(A70)')args(4)
        read(1,'(A70)')args(5)
        read(1,'(A70)')args(6)
      endif
      do 18 i=1,6
      ipt=0
      call getnum(work(i),iwork(i),args(i),70,ipt,iterm,ier)
      if(ier.ne.0)then
        write(6,*) 'input error, not a number:', args(i)
        STOP
      endif
  18  continue
      NX= iwork(1)
      NY= iwork(2)
      XMIN= work(3)
      XMAX= work(4)
      YMIN= work(5)
      YMAX= work(6)

      write(6,'(A2,2I6)') '#',NX,NY
      write(6,'(A2,4F14.4)') '#',XMIN, XMAX, YMIN, YMAX
      call buildxy(NX, NY, XMIN, XMAX, YMIN, YMAX)

      end
C ------------------------------------------------ SUBROUTINE buildxy
      subroutine  buildxy(NX, NY, XMIN, XMAX, YMIN, YMAX)
      implicit NONE
      integer NX,NY
      real XMIN, XMAX, YMIN, YMAX
      integer A(NX*NY)
C If your Fortran implementation does not support automatic arrays
C replace the above by a declaration of a sufficiently large
C permanent array, e.g.
C     integer A(16384)
      real WORK(6)
      character*80 STRING1
      integer  itxtostr
      external itxtostr

      integer I, J, NLINE, JX, JY, II, LU, IDUM, IPT, ITERM, IER
      I=0
      do 40 JY=1,NY
      do 40 JX=1,NX
      I=I+1
  40  A(I)=0
  
      JX=1
      JY=2
      LU=5
      NLINE=0
  74  continue
        READ(LU,'(A)',ERR=9999, END=747) STRING1
        IF(STRING1(1:1).EQ. '#') goto 74
        IF(STRING1(1:3).EQ. 'end') goto 747
        IF(STRING1(1:4).EQ. ' end') goto 747
        IPT=0
        I=0
  742   CONTINUE
        I= I+1
        CALL GETNUM(WORK(I),IDUM,STRING1,80,IPT,ITERM,IER)
        IF(IER.eq.0)goto 742
        if(I.le.JX .or. I.le.JY)then
          WRITE(6,*)'# header:'//STRING1(1:itxtostr(STRING1,STRING1))
          goto 74
        endif
        I=NINT((WORK(JX)-XMIN)*(NX-1)/(XMAX-XMIN))
        J=NINT((WORK(JY)-YMIN)*(NY-1)/(YMAX-YMIN))
C events that are off-limits are put at the edges:
        IF(I.lt.0)I=0
        IF(I.ge.NX)I=NX-1
        IF(J.lt.0)J=0
        IF(J.ge.NY)J=NY-1
        II=J*NX+I+1
        IF(II.gt.0 .and. II.LE.NX*NY)A(II)=A(II)+1
        NLINE=NLINE+1
        GOTO 74
C come here at end-of-file
  747   CONTINUE
        WRITE(6,*) '# ',NLINE, ' data points processed'
        call WRTA(6,A,NX*NY)
        return

 9999   WRITE(6,*) 'READ ERROR'
        END
C --------------------------------------------------- SUBROUTINE WRTA
      SUBROUTINE WRTA(LU, A, NXY)
      IMPLICIT NONE
      INTEGER LU
      INTEGER NXY
      INTEGER A(NXY)
      
      INTEGER N1, N2, K, K0, K1, IX
      CHARACTER*6 FORMX
      CHARACTER*128 FORMXY

      K1=0
  98  CONTINUE
      K0=K1+1
      K1=K1+20
      IF(K1.GT.NXY)K1=NXY
      FORMXY= '('
      N1=1
      do 999 K=K0, K1
      IX= A(K)
      IF(IX.LT.100) THEN
        FORMX='I3'
        N2=2
      ELSE IF(IX.LT.1000) THEN
        FORMX='I4'
        N2=2
      ELSE
        FORMX='I10'
        N2=3
      ENDIF
      FORMXY= FORMXY(1:N1)//FORMX(1:N2)
      N1= N1+N2
      IF(K.LT.K1)THEN
        FORMXY= FORMXY(1:N1)//','
        N1=N1+1
      ENDIF
  999 CONTINUE
      FORMXY= FORMXY(1:N1)//')'
      WRITE(LU,FORMXY) (A(K), K=K0,K1)
      IF(K1.LT.NXY)goto 98
      END
