      PROGRAM PGDEM3
C-----------------------------------------------------------------------
C Demonstration program for PGPLOT vector field plot.
C-----------------------------------------------------------------------
      INTEGER PGOPEN
      WRITE (*,'(A)') ' Demonstration of routine PGVECT'
C
      CALL SETPGFIL('vectorplot.ps')
      CALL PGEX35
      CALL PGEND
C-----------------------------------------------------------------------
      END

*Module: PGVECT -- vector map of a 2D data array, with blanking
*------------------------------------------------------------------------
*
*      SUBROUTINE PGVECT (A, B, IDIM, JDIM, I1, I2, J1, J2, C, NC, TR,
*     1                   BLANK)
*      INTEGER IDIM, JDIM, I1, I2, J1, J2, NC
*      REAL    A(IDIM,JDIM), B(IDIM, JDIM), TR(6), BLANK, C
*
*Draw a vector map of two arrays.  This routine is similar to
*PGCONB in that array elements that have the "magic value" defined by
*the argument BLANK are ignored, making gaps in the vector map.  The
*routine may be useful for data measured on most but not all of the
*points of a grid. Vectors are displayed as arrows; the style of the
*arrowhead can be set with routine PGSAH, and the the size of the
*arrowhead is determined by the current character size, set by PGSCH.
*
*Arguments:
* A      (input)  : horizontal component data array.
* B      (input)  : vertical component data array.
* IDIM   (input)  : first dimension of A and B.
* JDIM   (input)  : second dimension of A and B.
* I1,I2  (input)  : range of first index to be mapped (inclusive).
* J1,J2  (input)  : range of second index to be mapped (inclusive).
* C      (input)  : scale factor for vector lengths, if 0.0, C will be
*                   set so that the longest vector is equal to the
*                   smaller of TR(2)+TR(3) and TR(5)+TR(6).
* NC     (input)  : vector positioning code.
*                   <0 vector head positioned on coordinates
*                   >0 vector base positioned on coordinates
*                   =0 vector centered on the coordinates
* TR     (input)  : array defining a transformation between the I,J
*                   grid of the array and the world coordinates. The
*                   world coordinates of the array point A(I,J) are
*                   given by:
*                     X = TR(1) + TR(2)*I + TR(3)*J
*                     Y = TR(4) + TR(5)*I + TR(6)*J
*                   Usually TR(3) and TR(5) are zero - unless the
*                   coordinate transformation involves a rotation
*                   or shear.
* BLANK   (input) : elements of arrays A or B that are exactly equal to
*                   this value are ignored (blanked).

      SUBROUTINE PGEX35
C-----------------------------------------------------------------------
C Program to demonstrate the use of PGVECT along with
C PGCONB by illustrating the flow around a cylinder with circulation.
C
C          NX      total # of axial stations
C          NY      total # of grid pts in y (or r) direction
C-----------------------------------------------------------------------
      INTEGER MAXX, MAXY
      PARAMETER(MAXX=200,MAXY=200)
      INTEGER NX, NY, I, J, NC
      REAL PI, A, GAMMA, VINF, XMAX, XMIN, YMAX, YMIN, DX, DY
      REAL CPMIN, R2, BLANK
      REAL CP(MAXX,MAXY),X(MAXX),Y(MAXY),U(MAXX,MAXY),V(MAXX,MAXY),
     1   PSI(MAXX,MAXY)
      REAL TR(6),C(10)
      REAL XCUR,YCUR
      DATA XCUR,YCUR /0, 0/
      PARAMETER (PI=3.14159265359)
      DATA BLANK/-1.E10/
      CHARACTER*1 cursor

      READ(5,*) NX,NY
      READ(5,*) ((U(I,J),I=1,NX),J=1,NY) 
      READ(5,*) ((V(I,J),I=1,NX),J=1,NY) 
      do J=1,NY
      do I=1,NX
      if(MOD(I,4).ne.0 .or. MOD(J,4).ne.0) then
      U(I,J)=BLANK
      V(I,J)=BLANK
      endif
      enddo
      enddo 
C
C grid to world transformation
C
      TR(1)=0.0
      TR(2)=1.0
      TR(3)=0.0
      TR(4)=0.0
      TR(5)=0.0
      TR(6)=1.0
C
      NC=0
      CALL OPENPGDIS
  34  CONTINUE
      CALL PGENV (0.0,float(NX+1),0.0,float(NY+1),1,0)
      CALL PGIDEN
C
C
C vector plot
C
      CALL PGSAH (2, 45.0, 0.7)
      CALL PGSCH (0.1)
      CALL PGVECT (U,V,MAXX,MAXY,1,NX,1,NY,4.0,0,TR,-1.E10)
      CALL PGSCH(1.0)
C
C finished
C
      if(NC .eq. 0) then
        CALL PGCURS(XCUR,YCUR,cursor)
        NC=1
        CALL OPENPGPLOT
        goto 34
      endif
      CALL CLOSEPGFIL
      CALL PGEND()
C----------------------------------------------------------------------
      END
