* ------------------------------------------------- subroutine PUTINSP
      subroutine putinsp(NSP, indx, value)
      IMPLICIT NONE
      integer NSP
      integer indx
      double precision value
C The output of RBSIM is made available through this routine.
C Purpose: to store 'value' in a spectrum, i.e. an array specified by
C the spectrum number, 'NSP', at array element 'indx'.

C special values used for NSP
C (only used if test output is requested
C  indx running from 1 to NSTEP+1 = 126 (?) )
C 4 : cross section
C 5 : variance of cross ection
C 6 : outgoing energy
C 7 : depth
C 8 : incoming cross section
C 9 : incoming variance
C 10 : incoming energy

C normal values used for NSP
C indx running from 0 t0 1023
C ND : the calculated spectrum
C NX : the depth scale

      INTEGER IE,IRESULT,IDEPTH,ITEST
      COMMON /spectra/ IE(0:1023),IRESULT(0:1023),IDEPTH(0:1023),
     +                 ITEST(0:1023,4:10)

      include 'rbspectr.com'

C
      IF(indx .lt. 0 .or. indx .gt. 1023)then
        write(6,*)'putinsp: illegal indx=',indx
        return
      endif
      IF(NSP .eq. ND)THEN
       IRESULT(indx) = value
      ELSE IF (NSP .eq. NX) THEN
       IDEPTH(indx) = value
      ELSE
        do 47 NSP=4,10
        IF(NSP .eq. NTST(NSP))then
         ITEST(indx, NSP)= value
        endif
   47   continue
      ENDIF
      END
* ------------------------------------------------- subroutine FGETVAR
      subroutine fgetvar(prompt,NSP,name,cdefault)
      IMPLICIT NONE
      character*(*) prompt
      integer NSP
      character*9 name
      integer cdefault
C get the name and address of an array to store the results.
C the address (NSP) is later on passed on to routine putinsp
C name and cdefault not used in this version

      NSP=0
      call intarg(NSP, prompt)
      end
* ---------------------------------------------------- subroutine CRLF
      subroutine crlf()
      IMPLICIT NONE
      write(*,74)
  74  format(1X)
      end
* --------------------------------------------------- subroutine WRITT
      subroutine writt(text)
      IMPLICIT NONE
      character*(*) text
C write text without terminating carriage return
      integer j

      do 88 j=LEN(text),2,-1
      if(text(j:j) .gt. ' ') goto 89
  88  continue
  89  continue
      write(*, fmt='(A,$)') text(1:j)
      end
* ---------------------------------------------------- subroutine OUT_
      subroutine out_(text)
      IMPLICIT NONE
      character*(*) text
C output a line of text, given in 'text'
      integer j

      do 88 j=LEN(text),2,-1
      if(text(j:j) .gt. ' ') goto 89
  88  continue
  89  continue
      write(*, fmt='(A)') text(1:j)
      end
* ------------------------------------------------- subroutine ERUIT_
      subroutine eruit_(text)
      IMPLICIT NONE
      character*(*) text
C take an error exit
      write(6,fmt='(A)') text
      STOP 'error exit'
      end
* ------------------------------------------------------ program TEST
      program test
      IMPLICIT NONE

      include 'rbsim.com'

      INTEGER IE,IRESULT,IDEPTH,ITEST
      COMMON /spectra/ IE(0:1023),IRESULT(0:1023),IDEPTH(0:1023),
     +                 ITEST(0:1023,4:10)

      character*80 outpfile
      character*3 yesorno, cursor
      real energy(1024), yield(1024)
      logical PLOTOPEN
      real YMAX, EJ, E1, XCUR, YCUR
      DATA XCUR, YCUR /0,0/
      INTEGER j, NN, k

      call SETPGFIL('rbsim.ps')
      PLOTOPEN= .false.

* April 2012: eliminate Fortran input
  10  continue
      CALL TXTARG(outpfile,'Name output file:')
*  10  write(*,*) 'name output file:'
*      read(5,'(A)') outpfile
      open(unit=8, file=outpfile, access='sequential', status='new',
     + err=11)
      goto 100

  11  continue 
      write(*,*) 'cannot open NEW file ',outpfile,'. Try again'
      goto 10

  100 continue

      do 120 j=0,1023
      IRESULT(j)=0
      ITEST(j,10)=0
  120 continue

      call RBSIM

C The results are now in the common arrays IDEPTH, IRESULT
C IRESULT should be directly comparable with an experimental
C spectrum.

      YMAX=0
      NN=0
      write(8,110)'energy','depth','yield'
 110  FORMAT(/A7,2(1X,A8))
      do 112 j=0,1023
      EJ = ESTEP*j + EJ0
      if(EJ .lt. EMIN .and. IRESULT(j) .eq. 0)then
        goto 112
      endif
      NN=NN+1
      energy(NN)=EJ*1000.0
      yield(NN)=IRESULT(j)
      WRITE(8,111) EJ*1000.0, IDEPTH(j), IRESULT(j)
 111  FORMAT(F7.1,2(1X,I8))
      if(EJ .gt. EMAX .and. IRESULT(j) .eq. 0) goto 144
      IF(IRESULT(j) .gt. YMAX) YMAX=IRESULT(j)
 112  continue
 144  continue

      if(TESTFLAG)then
        write(8,146)'Xsection','variance','E_out','depth','Xsect_in',
     +              'Var_in', 'E_in'
 146    FORMAT(//7(1X,A8))
C 4 : cross section
C 5 : variance of cross ection
C 6 : outgoing energy
C 7 : depth
C 8 : incoming cross section
C 9 : incoming variance
C 10 : incoming energy
        do 147 j=0,1023
        write(8,148)(ITEST(j,k),k=4,10)
        if(ITEST(j,10).eq.0 .and. j.gt.0) goto 149
  147   continue
  148   FORMAT(7(1X,I8))
  149   continue
      endif

      E1 = (ESTEP*1024 + EJ0)*1000.0

      CALL OPENPGPLOT
      if(.not. PLOTOPEN)then
        CALL PGASK(.FALSE.)
        CALL PGPAGE()
        CALL PGSVP(0.1, 0.8, 0.1, 0.8)
        CALL PGSWIN(0.0,E1,0.0,YMAX*1.2)
        CALL PGBOX('BCINTS',0.0,0, 'BCINTSV',0.0,0)
        CALL PGIDEN
      endif
      CALL PGLINE(NN, energy, yield)

      CALL OPENPGDIS
      if(.not. PLOTOPEN)then
        CALL PGASK(.FALSE.)
        CALL PGPAGE()
        CALL PGSVP(0.1, 0.8, 0.1, 0.8)
        CALL PGSWIN(0.0,E1,0.0,YMAX*1.2)
        CALL PGBOX('BCINTS',0.0,0, 'BCINTSV',0.0,0)
        CALL PGIDEN
      endif
      CALL PGLINE(NN, energy, yield)
      WRITE(6,'(''==> Click mouse in PGPLOT Window <=='')')
      CALL PGCURS(Xcur,Ycur,cursor)
      PLOTOPEN= .true.

* April 2012: eliminate Fortran input
      CALL TXTARG(yesorno,'continue, yes or no?')
*      write(*,*) 'continue, yes or no?'
*      read(5,'(A)') yesorno
      if(yesorno(1:1).eq. 'y' .or. yesorno(1:1).eq.'Y')goto 10
      CALL CLOSEPGFIL
      CALL PGEND()
      end
