      SUBROUTINE XTALGEN(S,SP,IXYZ,F,NATOM,NATOM2,NIMP,NL,IDENT)
      REAL S(3,NATOM)
      REAL SP(3,NATOM2)
      REAL F(3,NATOM)
      INTEGER IXYZ(3,NIMP)
      CHARACTER*(*) IDENT

C FIND THE PROJECTED POSITIONS OF LATTICE ATOMS AND IMPURITIES
C IN A CRYSTAL.

C ARRAYS S and SP CONTAIN THE POSITIONS OF THE LATTICE ATOMS
C WITHIN AN ELEMENTARY CELL.

C ARRAY F DEFINES REFLECTION OPERATORS THAT APPLY TO THE UNIT CELL
C WHEN ITS ORIGIN IS SHIFTED FROM (0,0,0) TO ANOTHER ATOM POSITION.

C ARRAY IXYZ CONTAINS EQUIVALENT DIRECTIONS OF AN IMPURITY
C WITH RESPECT TO A PURE LATTICE POSITION.
C E.G. THE TRIPLET (+1,-3,-2) MEANS THAT A SHIFT (DX,DY,DZ) =
C (X1,-X3,-X2) IS EQUIVALENT WITH (DX,DY,DZ) = (X1,X2,X3).

      PARAMETER (MAXIND=3)
      PARAMETER (MAXNL=2)
      PARAMETER (MAXSITE=1000)
C MAXIND IS THE HIGHEST ORDER OF PLANES WHOSE PROJECTION IS PLOTTED.

C INPUT:
C A    : LATTICE CONSTANT
C XDEL,YDEL,ZDEL : IMPURITY SHIFT WITH RESPECT TO (0,0,0) POSITION
C                  IN DIAMOND (+D,+D,+D) IS AWAY FROM NEAREST NEIGHBOUR
C RB   : RADIUS OF PLOTTED CIRCLES (IN SAME UNITS AS A)
C SX   : PARAMETER FOR PERSPECTIVIC PROJECTION. NORMALLY ZERO.
C H,K,L: INDICES OF PLANE OF PROJECTION.

C OUTPUT:
C 1. PLOT OF ALL PURE SITES WITHIN A RADIUS OF A.
C    IMPURITY SITES WITH RESPECT TO THE (0,0,0) POS. ONLY.
C 2. LIST OF COORDINATES IN THE PROJECTION OF THOSE SITES,
C    TOGETHER WITH THEIR FREQUENCY OF OCCURENCE.
C NOTE. THE FREQUENCY OF ALL PURE SITES SHOULD BE THE SAME.
C    DUE TO THE FINITE SIZE OF THE PROJECTED CRYSTAL,
C    THIS MAY NOT ALWAYS BE FOUND.

      REAL H,K,L,H2,K2,L2,R(3,3),SITE(6,MAXSITE,MAXNL)
      REAL RZP(16,MAXSITE,MAXNL),XYZ(3),XYZP(3),XDEL(3)
      REAL YSITE(3,MAXSITE)
      CHARACTER*80 STRING
      COMMON /COMBOL/ SX,RB,A,B,C,PGRATIO
      INTEGER KS(MAXNL)
      DATA BUNIT,CUNIT/1,1/

      OPEN(UNIT=7,FILE='xtal',STATUS='UNKNOWN')
      REWIND 7
      KK=2
      CALL ASKPGFIL()
      WRITE(6,*)'text output to file xtal'

  1   WRITE(6,11)
  11  FORMAT(' LATTICE CONSTANT A, IMPURITY SHIFT XDEL,YDEL,ZDEL')
      READ(5,*)A,XDEL
      if(A.le.0.0) then
       CALL CLOSEPGFIL
       CALL PGEND()
       STOP
      ENDIF
      PGRATIO= 10.0/A
      WRITE(6,12)
  12  FORMAT(' RB=RADIUS OF ATOMS,SX=PERSPECTIVE ANGLE',
     + ' (0=NO PERSPECTIVE), H, K, L')
      READ(5,*)RB,SX,RH,RK,RL
      SX=SX/A
      B=A * B UNIT
      C=A * C UNIT
      WRITE(7,13)A,B,C,XDEL,RH,RK,RL
  13  FORMAT(' LATTICE DIMENSIONS:',3F9.5/
     +       ' IMPURITY SHIFT    :',3F9.5/
     +       ' PROJECTION        :',3(F4.0,5X))
C ATOMS WITHIN RADIUS SQRT(AP) ARE TO BE PLOTTED:
      AP=(A**2 + B**2 + C**2) / 2.0
      H=RH/A
      K=RK/B
      L=RL/C
C N-PLOT
      WRITE(6,15)
  15  FORMAT(' Hardcopy? 0=no, 1=yes:')
      READ(5,*)NOVERPL
C INTERPLANAR DISTANCE:
      D =1.0/SQRT(H**2+K**2+L**2)
      DO 20 I=1,3
      DO 20 J=1,3
      R(I,J)=0.0
  20  R(J,J)=1.0
      PHI0=45.0
      SKH=SQRT(K**2+H**2)
      IF(K.EQ.0.0 .AND. H.EQ.0.0)GOTO 21
      PHI0=-90.0
C ROTATE ABOUT Z-AXIS, ANGLE ARCTAN (K/H)
      CALL ROT(R,3,K/SKH,H/SKH)
C ROTATE ABOUT NEW Y-AXIS, ANGLE ARCCOS (L*D/C)
  21  CALL ROT(R,2,SKH*D,L*D)
C NOW THE NEW Z-AXIS HAS DIRECTION COSINES H*D/A, K*D/B, L*D/C
      PHI=PHI0*3.14159265/180.0
      SPHI=SIN(PHI)
      CPHI=COS(PHI)
C ROTATE AN 'ARBITRARY' ANGLE PHI0 ABOUT NEW Z-AXIS.
      CALL ROT(R,3,SPHI,CPHI)
      WRITE(7,66)((R(I,J),J=1,3),I=1,3)
  66  FORMAT(' ROTATIE-MATRIX:',3(/1X,3F12.7))
      DO 597 IFLAG=0,1
      IF(IFLAG.EQ.1)THEN
        IF (NOVERPL.LE.0.OR.KK.LT.2)GOTO 597
        CALL OPENPGPLOT
      ELSE
        if(KK/2*2 .eq. 0)goto 597
        CALL OPENPGDIS
      ENDIF
      CALL PGASK(.FALSE.)
      CALL PGPAGE
      CALL PGVSIZ(1.0, 5.0, 1.0, 5.0)
      RM=SQRT(AP)*1.1
      RPLOT=RM*1.3
      CALL PGSVP(0.25, 0.75, 0.2, 0.7)
      CALL PGSWIN(-RPLOT,RPLOT,-RPLOT,RPLOT)
      CALL PGWNAD(-RPLOT,RPLOT,-RPLOT,RPLOT)

      DO 112 LL=1,MAXNL
      DO 111 J=1,MAXSITE
      DO 110 I=1,6
  110 SITE(I,J,LL)=0
      DO 111 I=1,16
  111 RZP(I,J,LL)=0
  112 CONTINUE
      WRITE(STRING,9103)XDEL            ! TEXT AT BOTTOM OF GRAPH
 9103 FORMAT('dx dy dz:',3F5.1)
      CALL PGMTXT('T', 4.0, 1.0, 1.0,STRING)
      WRITE(STRING,9106)IDENT,RH,RK,RL
      CALL PGMTXT('T', 6.0, 1.0, 1.0,STRING)
 9106 FORMAT(A,'  h  k  l :',3F6.2)
      IPEN=3
      DO 22 J=0,360                    ! WITH 1 DEGREE DIVISIONS
      RR=1.04
      IF(J/10*10.EQ.J)RR=1.07
      P=3.141593*J/180.0
      RX=RM*SIN(P)
      RY=RM*COS(P)
      IF(IPEN.EQ.3) THEN
        CALL PGMOVE(RX,RY)
      ELSE
        CALL PGDRAW(RX,RY)
      ENDIF
      IPEN=2
      CALL PGDRAW(RR*RX,RR*RY)
      CALL PGDRAW(RX,RY)
  22  CONTINUE
      DO 212 LL=1,NL
      KS(LL)=0
  212 CONTINUE
      NS=0
C IN THE FOLLOWING LOOP THE CONSTRUCTED ROTATION MATRIX IS
C APPLIED TO A WHOLE SET OF SITES. RESULTS ARE SORTED IN
C ARRAYS SITE AND YSITE.
      NMIN=3
      NPLUS=+2
      DO 30 M=1,NATOM+NATOM2
      DO 30 NX=0,NMIN+NPLUS
      IF(M.LE.NATOM) THEN
          LL=1
          S1M=S(1,M)
          S2M=S(2,M)
          S3M=S(3,M)
      ELSE
          LL=2
          S1M=SP(1,M-NATOM)
          S2M=SP(2,M-NATOM)
          S3M=SP(3,M-NATOM)
      ENDIF
      IX=NX
      IF(IX.GT.NPLUS)IX=NPLUS-IX
      X=(IX+S1M)*A
      DO 30 NY=0,NMIN+NPLUS
      IY=NY
      IF(IY.GT.NPLUS)IY=NPLUS-IY
      Y=(IY+S2M)*B
      DO 30 NZ=0,NMIN+NPLUS
      IZ=NZ
      IF(IZ.GT.NPLUS)IZ=NPLUS-IZ
      Z=(IZ+S3M)*C
      XP=R(1,1)*X + R(1,2)*Y + R(1,3)*Z
      YP=R(2,1)*X + R(2,2)*Y + R(2,3)*Z
      ZP=R(3,1)*X + R(3,2)*Y + R(3,3)*Z
      IF(XP**2 + YP**2 .GT. AP ) GOTO 30
C GO SEE IF SITE WITH SAME XP,YP WAS FOUND BEFORE.
      DO 133 IS=1,KS(LL)
      DX=XP-SITE(4,IS,LL)
      DY=YP-SITE(5,IS,LL)
      IF((DX**2+DY**2).LT. 0.001*AP)THEN
C YES! UPDATE WEIGHT FOR THIS SITE
          SITE(6,IS,LL)=SITE(6,IS,LL)+1.0
          IF(SX.NE.0)GOTO 134          ! PLOT THIS IF PERSPECTIVE NEEDED
          GOTO 136
      ENDIF
  133 CONTINUE
C NO! ENTER A NEW SITE:
      IF(KS(LL).GE.MAXSITE)GOTO 134
      KS(LL)=KS(LL)+1
      IS=KS(LL)
      SITE(1,IS,LL)=X/A
      SITE(2,IS,LL)=Y/B
      SITE(3,IS,LL)=Z/C
      SITE(4,IS,LL)=XP
      SITE(5,IS,LL)=YP
      SITE(6,IS,LL)=1.0
  134 CALL BPLOT(XP,YP,ZP,M-NATOM)
  136 CONTINUE
      LZ=NINT(SITE(6,IS,LL))
      IF ( LZ.LE.16 ) RZP(LZ,IS,LL)=ZP
C SEE IF PROJECTION IS (0,0). IF SO GO THROUGH ALL IMPURITY SITES.
      IF(M.LE.NATOM .AND. XP**2+YP**2 .LT. 0.001*AP)THEN
          DO 26 JY=1,NIMP
          I1=IABS(IXYZ(1,JY))
          I2=IABS(IXYZ(2,JY))
          I3=IABS(IXYZ(3,JY))
          S1=FLOAT(I1/IXYZ(1,JY))*XDEL(I1) * F(1,M)
          S2=FLOAT(I2/IXYZ(2,JY))*XDEL(I2) * F(2,M)
          S3=FLOAT(I3/IXYZ(3,JY))*XDEL(I3) * F(3,M)
          XPP=R(1,1)*S1 + R(1,2)*S2 + R(1,3)*S3
          YPP=R(2,1)*S1 + R(2,2)*S2 + R(2,3)*S3
          ZPP=R(3,1)*S1 + R(3,2)*S2 + R(3,3)*S3
          DO 233 IS=1,NS
          DX=XPP-YSITE(1,IS)
          DY=YPP-YSITE(2,IS)
          IF(ABS(DX)+ABS(DY).LT. 0.002)THEN
              YSITE(3,IS)=YSITE(3,IS)+1.0
              GOTO 236
          ENDIF
  233     CONTINUE
          NS=NS+1
          IF(NS.GT.MAXSITE)GOTO 235
          YSITE(1,NS)=XPP
          YSITE(2,NS)=YPP
          YSITE(3,NS)=1.0
  235     CALL MPLOT(XP+XPP,YP+YPP,ZP+ZPP)
  236     CONTINUE
  26      CONTINUE
      ENDIF
  30  CONTINUE
      IF(IFLAG.EQ.0)WRITE(7,207)
  207 FORMAT('    PLANE      PHI')
      DO 210 IX=-MAXIND,MAXIND
      DO 210 IY=-MAXIND,MAXIND
      DO 210 IZ=-MAXIND,MAXIND
      DO 202 LD=2,MAXIND               ! FIND LARGEST COMMON DIVISOR
      IF(MOD(IX,LD).EQ.0 .AND. MOD(IY,LD).EQ.0 .AND.
     +   MOD(IZ,LD).EQ.0) GOTO 210
  202 CONTINUE
      H2    =FLOAT(IX)/A
      K2    =FLOAT(IY)/B
      L2    =FLOAT(IZ)/C
      RINPROD=H*H2 + K *K2 + L *L2
      IF (ABS(RINPROD)*A*B .GE. 0.003) GOTO 210
      XYZ(1) = K*L2 - L*K2             ! OUTER PRODUCT = INTERSECTION
      XYZ(2) = L*H2 - H*L2
      XYZ(3) = H*K2 - K*H2
      DO 209 J=1,3
      XYZP(J)=R(J,1)*XYZ(1)+R(J,2)*XYZ(2)+R(J,3)*XYZ(3)
  209 CONTINUE
      PHIE=ATAN2(XYZP(2),XYZP(1)) *180.0/3.14159265
      IF(IFLAG.EQ.0)WRITE(7,211) IX,IY,IZ,PHIE
  211 FORMAT(1X,3I4,F8.2)
      CALL PLAN(XYZP,IX,IY,IZ,RM,PHIE)
  210 CONTINUE
      CALL FSLEEP(2)
  597 CONTINUE                        ! END LOOP IFLAG
      DO 863 LL=1,NL
      WRITE(7,81)LL
      WRITE(7,86)
      DO 862 J=1,KS(LL)
      LZ=NINT(SITE(6,J,LL))
      IF(LZ.GT.16)LZ=16
      CALL SORT(RZP(1,J,LL),LZ)
      WRITE(7,861)(SITE(I,J,LL),I=1,6),(RZP(I,J,LL),I=1,LZ)
 862  CONTINUE
      IF(KS(LL).GE.MAXSITE)WRITE(7,82)
 863  CONTINUE
 86   FORMAT('    X     Y     Z     XP      YP   WEIGHT    ZP')
 861  FORMAT(1X,3F6.2,2F8.4,F4.0,2X,11F8.4,(/41X,11F8.4))
      WRITE(7,83)
      NS1=NS
      IF(NS1.GT.MAXSITE)NS1=MAXSITE
      WRITE(7,87)((YSITE(I,J),I=1,3),J=1,NS1)
      IF(NS.GT.MAXSITE)WRITE(7,82)
  81  FORMAT(' PURE LATTICE SITES,',I2,':')
  82  FORMAT(' ETCETERA............')
  83  FORMAT(' IMPURITY SITES:')
  87  FORMAT('      XP       YP     WEIGHT'/
     +  50(1X,2F9.5,2X,F7.1/))
      GOTO 1
      END
