C ===---===---===---===---===---===---===---===---= MAIN PROGRAM YIMP
      PROGRAM YIMP
      IMPLICIT NONE

C PROGRAM TO CALCULATE IMPURITY YIELD
C TAKES OUTPUT FROM FLUX  AS INPUT (LU=10)
C
C This is Ulrich Wahl's extended program YIMP, with automatic
C generation of impurity angular scans and raster scans, for most
C 'standard' lattice sites.
C Modified for use with flux version 6 by Peter Smulders, April 1996.
C The results of this program should be the same as those of 'yimp7'
C but the input is much more convenient (for zincblende and bcc structures).
C
Ccc comments by U. Wahl:
C MODIFIED FOR USE ON PC (MS-FORTRAN) BY U.WAHL
C  last change: 16.2.96
C
C     YIMP should be linked with the subroutines in LIBFLUX,
C     and SITEDATA
C
C
C CHANGES TO THE ORIGINAL SMULDERS/BOERMA VERSION: 
C     1) THE VERSION NUMBER WAS CHANGED TO V3.5. THIS VERSION IS 
C        OPTIMIZED TO HANDLE ZnS-STRUCTURES, BUT ALSO BCC STRUCTURES.
C     2) INCLUDE STATEMENT FOR 'FLUX3COM.FOR'
C     3) "CALL YIMPINIT, BYIMPINIT OR  YIMBINIT" DEFINES THE LOGICAL UNITS 
C        FOR INPUT/OUTPUT
C     4) SOME EXPLAINING REMARKS WERE INTRODUCED
C     5) THE INPUT OF COMMANDS AND DATA FROM THE KEYBOARD OR EXTERNAL FILES 
C        WAS CHANGED COMPLETELY AND IS NOW OPTIMIZED 
C        TO HANDLE BATCH JOBS!!
C     6) LATER ON AN OUTPUT-OPTION TO UNFORMATTED LIBRARY FILES WAS ADDED. 
C        THIS IS INTENDED TO BE USED IN CONJUNCTION WITH RASTER
C        SIMULATIONS.
C        NOTE THAT THE UNFORMATTED OUTPUT DOES NOT WORK FOR THE 
C        BATCH JOB VERSION OF THE PROGRAM.
C
Ccc comments by Peter Smulders (April 1996):
C
C Changes to the Wahl version:
C     1) updated the version number to 6.21
C     2) replaced keyboard input routines again. Some changes:
C        RREAL4 RINT4 and CHOOSE have modified calling sequence.
C        RREAL4 routine now returns 500 (was 0.05 (!)) when 
C        500 is given as input.
C        'Type ahead' is supported. 
C        Subroutine CHOOSE now reads type character.
C     3) So called 'Batch mode' not implemented. The keyboard input
C        routines are (as before) capable of handling
C        batch jobs as well as interactive jobs. 
C     4) fixed IOUTMODE to 1 (ASCII output file)
C        Commented out question asking to choose between 1 and 2,
C        but left everything else intact.
C     5) Modified calls to subroutines FINDFLX, GETFLX
C     6) Program reads the flux file, to get details on the host
C        lattice, etc.
C     7) Program no longer asks for Z_imp, after all these years :-)
C     8) 'Negative Debye temperature' interpreted as -u1, not -u2!!!
C
C     At startup the program tries to read file 'yimp.ini' in the
C     current directory. This file has the same layout as batflux.ini
C     line 1: name of input file for flux
C     line 2: name of output file of flux (FLXFILE)
C     line 3: name of output file of flux (LOGFILE)
C     line 4: name of output file of yimp (YLDFILE)
C     Only lines 2 and 4 are used in the current program.
C     If yimp.ini is not found, or the relevant lines are empty,
C     FLXFILE and YLDFILE are asked via the keyboard.
C     If YLDFILE is an existing file, new output is appended,
C     else the file is created.
C Dec. 1997:
C     - adapted to FLUX7
C     - lifted the calculation of the yield out of the main program
C       as separate subroutine 'yimp1', shared by the present program
C       and program yimp7.
C     - implicit none

      include 'FLUX7.COM'
      include 'YIMP.COM'

      REAL AVXSQ
      EXTERNAL AVXSQ

C Local Variables:

      REAL AREL, VIB(NVIBMAX)
      INTEGER NBIN1,NX,NY,NV1,NL1,ILAY,IWEIGHT,IOS,LTFLUX
      INTEGER IVIB,NVIB,IMARK,ICONF,INIT,KASE, VERSLEN
      INTEGER I,J,K,L,M,N,LL,JJ,IANGLE,IFAC,NHOST,NAXIS
      REAL OFAC,UJ1,UJ2,UI1SQ,UI2SQ,UI2,UI1
      REAL OLDT0, OLDTEMP
      REAL YI,XI
      REAL THETA, PHI, YIELD
      REAL VELO(1)

      CHARACTER*160 FLKOP
      CHARACTER*28 VERSY,OLDVERS
C VERSY and OLDVERS are strings read from the flux file
C common variable VERSION is set by the present program
      CHARACTER*5 FLUXID,YIMPID
      CHARACTER*20 TIME
      CHARACTER*100 F2DUM
      LOGICAL FEXIST

      CHARACTER*4 HOST
      CHARACTER*4 CHOICE(0:IAUTOMAX)
      CHARACTER*1 CHUNIT(0:2)

      SAVE OLDT0, OLDTEMP, OLDVERS

      INTEGER iquit,icontinue,ivector,isite,iSi,iIIIV,ibcc
      PARAMETER(iquit=0,
     1 icontinue=1,
     2 ivector=2,
     3 isite=3,
     4 iSi=4,
     5 iIIIV=5,
     6 ibcc=6)

      DATA CHOICE/
     +  'STOP',
     1  'EOF',
     2  'VEC',
     3  'SITE',
     4  'SI',
     5  'IIIV',
     6  'BCC'/
      DATA OLDVERS/' '/
      DATA YIMPID/'YIMP '/
      DATA AREL /1.0/
      DATA VIB /917.,99*0./
      DATA CHUNIT/'A','U','R'/
C------------------------------------------------------------------------
C Setting the flag IAUTO.LE.0 produces compiled versions which are to be 
C used for Batch jobs only, [not implemented in this version!!]
C IAUTO.GE.0 produces the interactive versions
C     IAUTO     Version or action:
C     0         Quit
C     1         Write the data to the specified disk file and continue
C     2         Calculation for several sites along a vector and 
C                 one vibration amplitude
C     3         Calculation for one site several vibration amplitudes
C     4         Calculation for all "Standard Sites" in Si
C     5         Calculation for all "Standard Sites" in III-V,
C               abbreviated as 3-5, ZnS-structure, e.g. InP, ZnSe etc.
C     6         Calculation for all "Standard Sites" in BCC

      VERSION(1:5)='V7.8'

C
C     IOUTMODE    Flag which sets the outputmode of calculated data
C     1           write to formatted ASCII-file ("*.YLD" mode)
C     2           write to binary file ("*.2DL" mode)

      IOUTMODE=1
      LBATCH= .FALSE.
      IAUTO=ivector
C
C in the "two-layer" version of FLUX the program computes the flux
C for one layer only. We call this layer 1, whether or not it was
C the first layer
      NLAY=1
C------------------------------------------------------------------------

C ICMINnnn(IAUTO)...first configuration for the "standard" lattice sites 
C                   to be considered for axis nnn, IAUTO see above
C ICMAXnnn(IAUTO)...last configuration for the "standard" lattice sites 

C Standard configurations in Si (Diamond-Structure)

      ICMIN100(iSi)=1
      ICMAX100(iSi)=   2
      ICMIN110(iSi)=1
      ICMAX110(iSi)=   6
      ICMIN111(iSi)=1
      ICMAX111(iSi)=   2
      ICMIN211(iSi)=1
      ICMAX211(iSi)=   6

C Standard configurations in 3-5 (ZnS-Structure)

      ICMIN100(iIIIV)=3
      ICMAX100(iIIIV)=   6
      ICMIN110(iIIIV)=7
      ICMAX110(iIIIV)=  18
      ICMIN111(iIIIV)=1
      ICMAX111(iIIIV)=   2
      ICMIN211(iIIIV)=7
      ICMAX211(iIIIV)=  18

C Standard configurations in BCC (e.g. Fe)

      ICMIN100(ibcc)=9
      ICMAX100(ibcc)=  11
      ICMIN110(ibcc)=19
      ICMAX110(ibcc)=  21
      ICMIN111(ibcc)=4
      ICMAX111(ibcc)=   6
      ICMIN211(ibcc)=0
      ICMAX211(ibcc)=   0

C In the following lines the default for often used values are placed:    

C In case of calculation of a 2D-library
C F2X1...x-start angle
C F2X1...x-stop angle
C F2XS...x-step angle
C F2X0...angle of x-origin of coordinate system
C F2Y1...y-start angle
C F2Y1...y-stop angle
C F2YS...y-step angle
C F2Y0...angle of y-origin of coordinate system

      F2X1= 0.
      F2X2= 2.
      F2XS=0.125
      F2Y1= 0.
      F2Y2= 2.
      F2YS=0.125
      F2X0=-2.
      F2Y0=-2.

C AIMP....mass in AMU of the impurity
C THIMP...Debye temperature of the impurity
C Default impurity is Li-8 in Si:

      AIMP=8.
      THIMP=917.

C NJUMPS..parameter only to be used if "jumping" (tunneling etc.) 
C         impurities are considered, =0 for all other uses.
C JANG....flag for the units in which displacements from the lattice 
C         sites are defined:
C         JANG=0: ANGSTROEMS
C              1: XYUN                       (units of flux cell)
C              2: R,PHI (polar coordinates in units of flux cell)
C NVIB....number of yield calculations with different vibration 
C         amplitudes to be done
C IVIB....loop parameter when calculating the above

      NJUMPS=0
      JANG=1
      NVIB=1
      IVIB=1
C ICONF...integer flag of the lattice site/displacement vector
C         (see subroutine SITEDATA for details)
C NSITES..number of equivalent impurity positions (projections) 
C         which belong to this specific impurity site
C NFAC....number of impurity sites which will be evaluated along 
C         the given displacement vector
C XYOFF(1,n)..x-offset of the displacement vector for site n 
C XYOFF(2,n)..y-offset of the displacement vector for site n 
C XY0(1,n)....x-projection of the displacement vector for site n 
C XY0(2,n)....y-projection of the displacement vector for site n 
C XYW(3,n)....x-projection of the displacement vector for site n 
C XYW(1,n) and XYW(2,n) are scratch arrays

C Default axis is <110> in Si, default lattice sites are the 
C substitutional site and the 4 displacements in the <111> direction:
                
      COMMENT='Si 110-Channel: displacement in <111> from BC to S'
      ICONF=1   
      NSITES=4
      NFAC=5

      XYOFF (1,1) = 0.
      XYOFF (2,1) = 0.
      XY0   (1,1) = 0.25
      XY0   (2,1) = 0.
      XYW   (3,1) = 2.

      XYOFF (1,2) = 1.
      XYOFF (2,2) = 1.
      XY0   (1,2) =-0.25
      XY0   (2,2) = 0.
      XYW   (3,2) = 2.

      XYOFF (1,3) = 0.5
      XYOFF (2,3) = 0.5
      XY0   (1,3) =-0.25
      XY0   (2,3) =-0.5
      XYW   (3,3) = 1.

      XYOFF (1,4) = 0.5
      XYOFF (2,4) = 0.5
      XY0   (1,4) = 0.25
      XY0   (2,4) = 0.5
      XYW   (3,4) = 1.

C------------------------------------------------------------------------
C IAUTO.GE.0: set the flags for INTERACTIVE processing and read the data  
C             for the yield calculation from the keyboard

c to get IOUTMODE = 2 back, uncomment the following 4 lines (PJMS Apr. 96)
c       WRITE(*,*)'Select the output-mode'
c       WRITE(*,*)'(1) formatted ASCII-file ("*.YLD"-mode)'
c       WRITE(*,*)'(2) unformatted binary file ("*.2DL" mode)'
c       CALL RINT4(IOUTMODE)

        IF(IOUTMODE.EQ.1) THEN
          CALL YIMPINIT
c get the AXIS value from the identifier of file COMBIN.FIG
c 3 digits starting from the first numerical character after pos. 3
          NAXIS=4
 304      IF(CONFIGID(NLAY)(NAXIS:NAXIS).GT.'9')THEN
            NAXIS=NAXIS+1
            IF(NAXIS.LT.8)goto 304
          ENDIF
          AXIS=CONFIGID(NLAY)(NAXIS:NAXIS+2)
        ELSE IF(IOUTMODE.EQ.2) THEN
          CALL YIMBINIT(FEXIST,INIT)

          IF(FEXIST.AND.INIT.EQ.0) THEN
          F2X2=F2X1+(NF2X-1)*F2XS
          F2Y2=F2Y1+(NF2Y-1)*F2YS
        ENDIF

        WRITE(*,*)'Give the x-start angle of the simulation'
        CALL RREAL4(F2X1,'F6.3')
        WRITE(*,*)'Give the x-stop angle of the simulation'
        CALL RREAL4(F2X2,'F6.3')
        WRITE(*,*)'Give the x-step angle'
        CALL RREAL4(F2XS,'F8.5')

        WRITE(*,*)'Give the y-start angle of the simulation'
        CALL RREAL4(F2Y1,'F6.3')
        WRITE(*,*)'Give the y-stop angle of the simulation'
        CALL RREAL4(F2Y2,'F6.3')
        WRITE(*,*)'Give the y-step angle'
        CALL RREAL4(F2YS,'F8.5')

        WRITE(*,*)'Give the x0-angle of the coordinate system'
        CALL RREAL4(F2X0,'F6.3')
        WRITE(*,*)'Give the y0-angle of the coordinate system'
        CALL RREAL4(F2Y0,'F6.3')

        NF2X=NINT(ABS((F2X2-F2X1)/F2XS))+1
        NF2Y=NINT(ABS((F2Y2-F2Y1)/F2YS))+1

        WRITE(*,*)'NF2X   = ',NF2X
        WRITE(*,*)'NF2Y   = ',NF2Y
      ELSE
        STOP 'illegal value for IOUTMODE'
      ENDIF

      VERSLEN=28
  160 VERSLEN= VERSLEN-1
      IF(VERSLEN.gt.1 .and. VERSION(VERSLEN:VERSLEN).le.' ')GOTO 160
      DO 162 L=1,2
      WRITE(LUN(L),161) YIMPID,VERSION(1:VERSLEN)
  161 FORMAT('   PROGRAM : ',A5,A/
     +       ' PROJECTED ATOM POSITIONS IN UNITS XUN,YUN')
       DO 162 K=1,NIND(NLAY)
        DO 163 LL=1,NLS(NLAY)
          ISPEC(LL)=0
         DO 163 J=0,MODZ(NLAY)-1
           IF(IATOM(J,LL,NLAY).EQ.K)ISPEC(LL)=ISPEC(LL)+1
  163   CONTINUE
        WRITE(LUN(L),164)XATOM0(K,NLAY),YATOM0(K,NLAY),ISPEC(1),ISPEC(2)
  164 FORMAT(' X=',F7.4,' Y=',F7.4,'   N(1)=',I2,'  N(2)=',I2)
  162 CONTINUE

1000  CONTINUE

      IF (IVIB.GT.NVIB.AND.LBATCH) GOTO 910

      IF (IAUTO.EQ.icontinue.OR.IAUTO.EQ.ivector
     +    .OR.(IAUTO.GE.isite.AND.IVIB.GT.NVIB)) THEN
        LAUTO=.FALSE.
170     WRITE(*,*)'(STOP) Stop the program'
        WRITE(*,*)'(EOF) write end-of-file record to disk file'
        WRITE(*,*)'    Do yield calculation for...'
        WRITE(*,*)'(VEC) several vector sites one vibration amplitude'
        WRITE(*,*)'(SITE) one site several vibration amplitudes'
        WRITE(*,*)'(SI) all standard sites in Si'
        WRITE(*,*)'(IIIV) all standard sites in 3-5 (III-V)'
        WRITE(*,*)'(BCC) all standard sites in BCC'
        CALL CHOOSE(IAUTO,CHOICE,IAUTOMAX)
        IF(IAUTO.GT.isite)THEN
          IF(IAUTO.EQ.iSi .or. IAUTO.eq.iIIIV) THEN
            HOST='GAAS'
            NHOST=4
          ELSE IF(IAUTO.eq. ibcc)THEN
            HOST='BCC '
            NHOST=3
          ENDIF
          IF(HOST.ne.CONFIGID(NLAY)(1:NHOST))THEN
            WRITE(6,363)HOST,CONFIGID(NLAY)(1:NHOST)
  363       FORMAT('>>>> WARNING: chosen lattice (',A4,
     +       ') conflicts with host type in simulation (',A4,')')
          ENDIF
        ENDIF

        IF (IAUTO.LT.0.OR.IAUTO.GT.IAUTOMAX) THEN
          GOTO 170
        ELSEIF (IAUTO.EQ.iquit) THEN
          GOTO 910
        ELSEIF (IAUTO.EQ.icontinue) THEN
          IF(IOUTMODE.EQ.1) THEN
            ENDFILE(7)
            REWIND(7)
171         READ(7,'(A1)',END=172)
            GOTO 171
172         BACKSPACE (7)
          ELSEIF(IOUTMODE.EQ.2) THEN
            ENDFILE(7)
            BACKSPACE (7)
          ENDIF

          IAUTO=ivector
          GOTO 1000
        ELSEIF (IAUTO.GE.isite.AND.IAUTO.LE.IAUTOMAX) THEN
          LAUTO=.TRUE.
        ENDIF


C       READ A and DEBYETEMP OF IMPURITY

        WRITE(*,*)'Give the impurity mass A'
        CALL RREAL4(AIMP,'F8.3')

        IF(IAUTO.EQ.isite) THEN
          NFAC=1
          WRITE(*,*)'Give NVIB   = number of vibration amplitudes '
          WRITE(*,*)'to be considered '
          CALL RINT4(NVIB)
        ELSE
          NVIB=1
        ENDIF
      
        DO 180 I=1,NVIB
178       FORMAT(' Vibration amplitude Nr. ',I3,' of ',I3)
          WRITE(*,178) I,NVIB
          WRITE(*,*)'Give the impurity DEBYE temperature (positive)'
          WRITE(*,*)'or thermal vibration amplitude -u1 (negative)'
          CALL RREAL4(VIB(I),'F8.4')
180     CONTINUE
        IVIB=1
      ENDIF

2000  CONTINUE

      KASE=0
      CALL SITEDATA(IMARK,ICONF)

      IF(IMARK.EQ.901) THEN
        GOTO 901
      ELSEIF(IMARK.EQ.905) THEN
        GOTO 905
      ENDIF

      IF (ICONF.EQ.0) THEN
        COMMENT='Configuration taken from manual input'
C READ NSITES AND NJUMPS ; IF NSITES AND NJUMPS =0 EXIT
        WRITE(*,*)'Give NSITES = number of equivalent impurity'
        WRITE(*,*)'positions which belong to this specific site'
        CALL RINT4(NSITES)
        WRITE(*,*)'Give NJUMPS = number of jumps for site-changing'
        WRITE(*,*)'positions (this number is 0 for Li-8)'
        CALL RINT4(NJUMPS)

        IF (NSITES .EQ. 0  .AND.  NJUMPS .EQ. 0) STOP

        WRITE(*,*)'Select the units for the input of the position data'
        WRITE(*,*)'(A) Angstroms'
        WRITE(*,*)'(U) (XY-units of flux cell)'
        WRITE(*,*)'(R) R,PHI (polar coordinates in Angstrom,deg)'
        CALL CHOOSE(JANG,CHUNIT,2)

C THE X- AND Y-IMPURITY ARE INCREMENTED BY XYOFF(1..2)
C THE XYOFF VALUES ARE NOT MULTIPLIED BY 'FACTOR'
C THEY ARE IN ANGSTROM FOR JANG=0 OR 2, IN XUN,YUN FOR JANG=1

C READ (XI,YI,WEIGHT) FOR I=1,NSITES
C

        DO 3 I=1,NSITES
        WRITE(*,9001) I
9001    FORMAT(' Considered site',I3)
        WRITE(*,*)'Give X0 = the original X coordinate of the site'
        CALL RREAL4(XYOFF(1,I),'F8.5')
        WRITE(*,*)'Give Y0 = the original Y coordinate of the site'
        CALL RREAL4(XYOFF(2,I),'F8.5')

        IF(JANG.NE.2) THEN
          WRITE(*,*)'Give DX = the X component of the displacement'
          WRITE(*,*)'          vector'
          CALL RREAL4(XY0(1,I),'F8.5')
          WRITE(*,*)'Give DY = the Y component of the displacement'
          WRITE(*,*)'          vector'
          CALL RREAL4(XY0(2,I),'F8.5')
        ENDIF
        IF(JANG.EQ.2) THEN
          WRITE(*,*)'Give R = the size of the displacement'
          WRITE(*,*)'         vector'
          CALL RREAL4(XY0(1,I),'F8.5')
          WRITE(*,*)'Give PHI = the azimuth of the displacement'
          WRITE(*,*)'          vector'
          CALL RREAL4(XY0(2,I),'F8.3')
        ENDIF

        WRITE(*,*)'Give WEIGHT = the statistical weight of this'
        WRITE(*,*)'              position '
        CALL RREAL4(XYW(3,I),'F8.5')

        VAL(1)=XYOFF(1,I)
        VAL(2)=XYOFF(2,I)
        VAL(3)=XY0(1,I)
        VAL(4)=XY0(2,I)
        VAL(5)=XYW(3,I)

  3     CONTINUE
      ENDIF

C ---------------------------------------------------------------
C THE FOLLOWING LOOP IS SKIPPED FOR NJUMPS=0
C
C READ(X1,X2,Y1,Y2,WEIGHT FOR J=1,NJUMPS. PRE-PROCESS.
      DO 4 J=1,NJUMPS
      CALL INREAL(VAL,7, 'X0,Y0,X1,Y1,X2,Y2,WEIGHT')
      DO 41 JJ=1,2
  41  STEP(JJ) = ( VAL(JJ+4) - VAL(JJ+2) ) * AREL / 7.0
      DO 44 N = -3,+3
      NSITES = NSITES + 1
      I = NSITES
      DO 42 JJ=1,2
      XYOFF(JJ,I)=VAL(JJ)
  42  XY0(JJ,I) = VAL(JJ+2) + FLOAT(2*N) * STEP(JJ)
      UJ1 = FLOAT(2*N-1) / 7.0
      UJ2 = FLOAT(2*N+1) / 7.0
  44  XYW(3,I)  = WJ * ( ASIN(UJ2) - ASIN(UJ1) ) / PI
   4  CONTINUE
C ---------------------------------------------------------------


C READ OVERALL FACTORS
     
      IF (IAUTO.EQ.ivector) THEN
        WRITE(*,*)'Give NFAC   = number of impurity sites which will be'
        WRITE(*,*)'evaluated along the displacement vector'
        CALL RINT4(NFAC)
      ELSE IF (IAUTO.EQ.isite)THEN
        NFAC=1
      ENDIF
      
      IF (IAUTO.EQ.ivector.OR..NOT.LAUTO) THEN
        DO 9010 I=1,NFAC
9005      FORMAT(' Displacement Nr. ',I3,' of ',I3)
          WRITE(*,9005) I,NFAC
          WRITE(*,*)'Give OFAC = the displacement in relative units'
          WRITE(*,*)'of the displacement vector'
          CALL RREAL4(DOFAC(I),'F8.5')
          IF(.NOT.LAUTO) LAUTO=.TRUE.
9010    CONTINUE
      ENDIF

      DO 900 IFAC=1,NFAC
      OFAC=DOFAC(IFAC)

C REWIND LU=10
      REWIND 10
      IANGLE = 0
C READ RECORD OF UNIT 10; IF END OF FILE GOTO LABEL 800
  5   CONTINUE
      CALL FINDFLX(10,NBIN1,NBINF,NX,NY,NV1,NL1,NLA,ILAY,IWEIGHT,IOS)
      IF(IOS.NE.0)GOTO 800
      IF(NL1.GT.ML      .OR.
     2   NX*NY.GT.NF*NF .OR.
     3   NBIN1.GT.NBIN  .OR.
     4   ILAY.GT.NLAYER)THEN
          IOS=255
          GOTO 800
      ENDIF
      CALL GETFLX(10,NBIN1,NX,NY,0,NL1,NLA,ILAY,IWEIGHT,
     6            FLKOP,RAAK,DAV,DDAV,FLUX,VELO)
C UNPACK FLKOP
      READ(FLKOP,6)FLUXID,VERSY,TIME,THETA,PHI,
     6 T0,Z1,Z2(1,NLAY),ZIMAX,ZIFWHM,NDELZ,UD(NLAY),
     7 XUN(NLAY),YUN(NLAY),TEMP,NTRX,LTFLUX
  6   FORMAT(A5,A27,A20,3F8.3,4F8.1,I6,F8.4,2F8.5,F8.2,I6,I4)
C DETERMINE THERMAL VIBRATION AMPLITUDE OF IMPURITY
      IF(VIB(IVIB).GT.0)THEN
          UI1SQ = AVXSQ(AIMP,VIB(IVIB),TEMP)
      ELSE
          UI1SQ = VIB(IVIB)*VIB(IVIB)
      ENDIF
      UI2SQ = 2.0 * UI1SQ
      UI2 = SQRT(UI2SQ)
      UI1 = SQRT(UI1SQ)
C MULTIPLY X,Y BY FACTOR
      DO 66 I=1,NSITES
      IF(JANG.NE.2)THEN
          XI = OFAC * XY0(1,I)
          YI = OFAC * XY0(2,I)
      ELSE
          XI = OFAC * XY0(1,I) * COS ( XY0(2,I) * DEGREE )
          YI = OFAC * XY0(1,I) * SIN ( XY0(2,I) * DEGREE )
      ENDIF
      XI=XI+XYOFF(1,I)
      YI=YI+XYOFF(2,I)
      IF ( JANG .EQ. 1 ) THEN
          XI = XI * XUN(NLAY)
          YI = YI * YUN(NLAY)
      ENDIF
      XYW(1,I)=XI
      XYW(2,I)=YI
   66 CONTINUE

C DO SOME OUTPUT IN FIRST ROUND

      IF(KASE.EQ.0 .OR. LATTICE(NLAY).NE.LTFLUX 
     +    .OR.VERSY(1:3).NE.OLDVERS(1:3)
     +    .OR.VERSY(5:28).NE.OLDVERS(5:28)
     +    .OR.T0.NE.OLDT0.OR.TEMP.NE.OLDTEMP)THEN

        DO 625 L=1,2
          WRITE(LUN(L),61)FLUXID,VERSY,TIME,T0,Z1,Z2(1,NLAY),
     +     XUN(NLAY),YUN(NLAY)
   61     FORMAT(' INPUT FROM: ',A5,A27,A20/
     +   ' PROJECTILE: E=',F8.3,' Z=',F8.3/
     +   ' LATTICE   : Z=',F8.3,' XUN=',F8.5,' YUN=',F8.5,' ANGSTR.')
          WRITE(LUN(L),62)AIMP,VIB(IVIB),ZIMAX,ZIFWHM,TEMP,UI1
   62     FORMAT(' IMPURITY  :  A=',F5.1,' DEBYETEMP=',F8.3/
     +           '           DEPTH (ANGSTR)=',F8.1,' FWHM=',F8.1/
     +           '           TEMPERATURE   =',F8.2,' U1  =',F8.5)
          WRITE(LUN(L),63) COMMENT
   63     FORMAT(' CONFIGURATION: ',A50)
          WRITE(LUN(L),67) UI2
   67     FORMAT('     U2  =',F8.5)
  625   CONTINUE

        IF(IOUTMODE.EQ.2) THEN
          IF(.NOT.FEXIST.OR.INIT.EQ.1) THEN

C File structure of *.2DL files is as follows
C record 1: 128 bytes file header:     
C           nf2x      2 integer*2
C           nf2y      6 real*4
C           f2x1
C           f2y1
C           f2xs
C           f2y2
C           f2x0
C           f2y0
C           F2DUM(1:100) 100 character*1 (blanks, reserved for later use)
C.......................................................................           
C record 2:  64 bytes spectrum header:
C           NF2      1 integer*2
C           COMMENT 50 character*1 
C           OFAC     2 real*4
C           UI2
C           F2DUM(1:4) 4 character*1 (blanks, reserved for later use)
C.......................................................................           
C record 3:  nf2x*nf2y real*4 bytes spectrum
C.......................................................................           

            DO 626 I=1,100
              F2DUM(I:I)=' '
626         CONTINUE
            WRITE(7)nf2x,nf2y,f2x1,f2y1,f2xs,f2ys,f2x0,f2y0,F2DUM

            INIT=0
            FEXIST=.TRUE.

          ENDIF
        ENDIF

        KASE=1
        IANGLE=0
        OLDT0=T0
        OLDTEMP=TEMP
      ENDIF
C TEST IF THE SAME COMMON BLOCK COMFIG WAS USED IN FLUX AND YIMP
C IF THE LATTICE TYPE DIFFERS A FATAL ERROR OCCURS
C IF THE VERSION IDENTIFIER DIFFERS ONLY A WARNING IS GIVEN.
      IF(LATTICE(NLAY).NE.LTFLUX)
     + STOP 'Non matching lattice types'

      IF (IANGLE .EQ. 0) THEN
          DO 65 L=1,2
          IF(VERSION(1:15).NE.VERSY(1:15).and. OLDVERS.NE.VERSY)
     +      WRITE(LUN(L),630)VERSY(1:15),VERSION(1:15)
          WRITE(LUN(L),635) OFAC
          WRITE(LUN(L),633)
          WRITE(LUN(L),64)((XYW(M,N),M=1,3),N=1,NSITES)
  630     FORMAT(5(1H>),' WARNING: NON-MATCHING FLUX:',A15,
     +    '/YIMP:',A15,5(1H<))
  633     FORMAT(' SITE(S)   : X(ANGSTR.) Y(ANGSTR.)     WEIGHT')
  635     FORMAT(20('*'),' FACTOR = ',F7.3,2X,20('*'))
   64     FORMAT( 100(12X,3F11.5/) )
   65     CONTINUE
          WRITE(6,68)
   68     FORMAT(' ANG  TEMP  TRACKS   THETA       PHI     YIELD ')
      ENDIF
      OLDVERS=VERSY

      CALL YIMP1(YIELD, FLUX,NX, NY, NSITES,
     +LATTICE(NLAY), MODX(NLAY), MODY(NLAY), XUN(NLAY), YUN(NLAY),
     + XYW, UI2SQ)

      IANGLE = IANGLE + 1

      IF(IOUTMODE.EQ.1
     +        .OR.IANGLE.EQ.1
     +        .OR.MOD(IANGLE,50).EQ.0 
     +        .OR.IANGLE.EQ.NF2X*NF2Y) 
     +        THEN
          WRITE (6,8) IANGLE,TEMP,NTRX, THETA,PHI,YIELD
      ENDIF
  8   FORMAT(I4,F6.0,I6,2F10.3,2F10.5)

      YYIELD(IANGLE)=YIELD
      IF(IOUTMODE.EQ.1) THEN
        WRITE(7,9) IANGLE,THETA,PHI,YIELD
  9     FORMAT(5HANGLE,I7,3F11.6)
      ENDIF

      GOTO 5
  800  CONTINUE
C     IOS=-1: END OF FILE: No more angles in this Flux file.
C     In case of binary output, check for correct number of angles,
C     and write header and yield to file
C     

      IF(IOS.EQ.-1)THEN
        IF(IOUTMODE.EQ.2) THEN
          IF (IANGLE.NE.NF2X*NF2Y) THEN
            WRITE(*,*)'IANGLE = ',IANGLE
            WRITE(*,*)'NF2X   = ',NF2X
            WRITE(*,*)'NF2Y   = ',NF2Y
            WRITE(*,*)'INFILE = ',INFILE
            WRITE(*,*)'YLDFILE= ',YLDFILE
            STOP 'Inconsistent number of angles in *.FLA-file'
          ENDIF
          NF2=NF2+1

          WRITE(*,*)'NF2    = ',NF2
          WRITE(*,*)'IANGLE = ',IANGLE
          WRITE(*,*)'NF2X   = ',NF2X
          WRITE(*,*)'NF2Y   = ',NF2Y

          WRITE(7)NF2,COMMENT,OFAC,UI2,F2DUM(1:4)
          WRITE(7) (YYIELD(I),I=1,IANGLE)
        ENDIF
      ELSE
          WRITE (6,1996) IOS
 1996     FORMAT(' ERROR NO',I4)
          STOP 'ERROR IN FLX FILE'
      ENDIF
      IF(IOUTMODE.EQ.1) THEN
        WRITE (7,92)
  92    FORMAT(1X)
        CALL SYNCF(7)
      ENDIF

900   CONTINUE

901   CONTINUE
      IF (IAUTO.EQ.ivector) GOTO 1000
      IF (IAUTO.EQ.isite) GOTO 905
      ICONF=ICONF+1
      GOTO 2000

905   CONTINUE
      IVIB=IVIB+1
      GOTO 1000

910   CLOSE(7)

      IF (LBATCH) THEN
        CALL LFOPEN(2,STATFILE,'OLD',0,0)
        READ(2,'(A70)') INFILE
        READ(2,'(A70)') YLDFILE
        READ(2,'(A70)') LOGFILE
        READ(2,'(A70)') YLDFILE
        READ(2,*) NSTART
        READ(2,*) NRSTART
        BACKSPACE(2)
        BACKSPACE(2)
        BACKSPACE(2)
        BACKSPACE(2)
        WRITE(2,'(A5)')'READY' 
        WRITE(2,'(A70)')YLDFILE 
        WRITE(2,*) NSTART
        WRITE(2,*) NRSTART
        CLOSE (2)
      ENDIF

      END

      BLOCK DATA
      include 'YIMP.COM'
      DATA DOFAC /0.,.25,.5,.75,1.,1.25,1.5,1.75,2.,91*0./
      END
