      PROGRAM DETTMAN
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      REAL ZION, A1, T1, B50, RNE, EPS
      REAL DELTAE,BA0,VV0
C THIS PROGRAM CALCULATES THE CORE CONTRIBUTION TO THE ELECTRONIC
C ENERGY LOSS BY A SUMMATION OVER THE SHELLS INVOLVED.
C FOR THE METHOD USED SEE COMMENT OF FUNCTION DELTAE.
C INPUT  ( EVERY ITEM ON A SINGLE LINE ) :
C  ZION
C  ION MASS (IN ATOMIC MASS UNITS)
C  ION ENERGY (MeV)
C  RANGE OF IMPACT PARAMETER B IN ANGSTROM.
C        (FOR GENERATING INPUT FOR PROGRAM FLUKS,
C        (THIS RANGE SHOULD BE CHOSEN EQUAL TO 2)
C  NE  : NUMBER OF ELECTRONS IN CURRENT (SUB)SHELL.
C        THE PROGRAM TERMINATES ON NE=0.
C  EPS : BINDING ENERGY IN eV
C  NEXT VALUE FOR NE, AND CORRESPONDING EPS, OR,
C  TERMINATING 0.
C
      PARAMETER(ABOHR=0.529177,RYDBERG=13.6058,AMU=931.5,
     $      HBAR=1.0545887D-34,EM=9.109534D-31,C=2.997925E8)
      DIMENSION DE(50) , DEE(50)
      DATA DE/50*0.0/
      CALL INREAL1(ZION,'Z ION')
      CALL INREAL1(A1,'ION MASS')
      CALL INREAL1(T1,'ENERGY(MeV)')
      CALL INREAL1(B50,'BMAX (ANGSTROM)')
  1   CALL INREAL1(RNE,'NR OF ELECTRONS')
      IF(RNE.LE.0)GOTO 300
      CALL INREAL1(EPS,'E-binding (eV)')
      ZI=SQRT(EPS/RYDBERG)
      A0=ABOHR/ZI
      V0C=HBAR/(A0*1.0D-10 * EM * C)
      AM1=A1*AMU
      VC=SQRT(T1* (T1+2.0*AM1) ) / (T1+AM1)
      VV0=VC/V0C
      WRITE(6,8)'EPS',EPS,'A1',A1,'T1',T1
      WRITE(6,8)'B50',B50,'ZI',ZI,'A0',A0
      WRITE(6,8)'V0C',V0C,'VC',VC,'VV0',VV0
  8   FORMAT(3(A10,': ',F10.5))
      CALL PRINTIME
      DO 50 I1=1,50
      IB=I1
      B=FLOAT(IB)*B50/50.0
      BA0=REAL(B/A0)
      DEE(IB)=RYDBERG*DELTAE(BA0,VV0)*RNE*ZION**2
      DE(IB) = DE(IB) + DEE(IB)
      IF(DEE(IB).LT.5.0D-6)GOTO 200
*      WRITE(6,48)IB,DEE(IB)
*  48  FORMAT(' IB=',I3,'  dE(eV)=',F12.5)
  50  CONTINUE
  200 CONTINUE
      CALL PRINTIME
      WRITE(6,320)
  320 FORMAT(/' CONTRIBUTION OF THIS SUBSHELL, IN eV')
      WRITE(6,330)(DEE(I1),I1=1,IB)
      GOTO 1
  300 CONTINUE
      WRITE(6,310)
  310 FORMAT(/' SUM OVER ALL SUBSHELLS, B= 1..50 *(BMAX/50)')
      WRITE(6,330)DE
  330 FORMAT(5F12.5)
      END
C -------------------------------------------------- FUNCTION DELTAE
      FUNCTION DELTAE(BA0,VV0)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      REAL DELTAE,BA0,VV0
CALCULATES THE EXPRESSION GIVEN BY EQ.39 OF DETTMAN AND ROBINSON,
C PHYS REV B,VOL 10 (1974),PAGE 1-9.
C GIVING THE ENERGY OF A PROTON-ATOM COLLISION.
C BA0 = B/A0, WHERE B=IMPACT PARAMETER, A0=SCALED BOHR RADIUS
C VV0 = V/V0, WHERE V=SPEED, V0=HBAR/(M*A0)
C DELTAE IS IN UNITS OF THE RYDBERG CONSTANT (13.6 eV).
C THE EXPRESSION OF D&R CONTAINS TWO SURFACE INTEGRALS. THESE
C ARE EVALUATED USING THEIR F(RHO,PHI), RATHER THEN RHO AS
C INTEGRATION VARIABLE. THIS RESULTS IN BOTH INTEGRALS HAVING
C AN INNER INTEGRAL OF THE FORM H(X,Y)= INTEGRAL XSI*K1(XSI)*DPSI
C WITH XSI=SQRT(X**2 + Y**2 - 2*X*Y*COS(PSI)).
C
      EXTERNAL GK0SQ,GK0
      COMMON /COMBES/ ALF,GAM,B,R,XX,YY,AA
       PARAMETER(PI=0.314159265358979D+01,TWOPI=2.0D0*PI)
      ALF=2.0*VV0
      B=BA0/VV0
COMPUTE INTEGRAL K0-SQUARED*H(ALF*XSI,ALF*B):
      YY=ALF*B
      AREA1=0.0
      DEL=0.3
      IF(DEL.GT.B*0.5)DEL=B*0.5
      EPS=2.0D-4
      RELERR=EPS
      Z=0.0
  10  A=Z
      Z=Z+DEL
          PREC=(Z-A)/1000.0
          CALL DWOMBERG(GK0SQ,A,Z,RELERR,AREA,PREC,H,KER)
          IF(KER.NE.0)CALL ERRMESS('DELTAE',KER)
      AREA1=AREA+AREA1
*     WRITE(6,*)A,Z,AREA,AREA1
      IF(Z.GT.B)THEN
          DEL=2.0/ALF
          IF(AREA.LT.AREA1*2.0D-6)GOTO 19
          RELERR=0.1*EPS*AREA1/AREA
      ENDIF
      IF(Z.LT.100.0)GOTO 10
 19   CONTINUE
COMPUTE INTEGRAL K0*H(GAM*XSI,GAM*B)
      AREA2=0.0
      GAMSQ=1.0+ALF**2
      GAM=SQRT(GAMSQ)
      YY=GAM*B
      DEL=0.25
      IF(DEL.GT.B*0.25)DEL=B*0.25
      Z=0.0
      EPS=1.0D-4
      RELERR=EPS
  20  A=Z
      Z=Z+DEL
      PREC=(Z-A)/1000.0
      CALL DWOMBERG(GK0,A,Z,RELERR,AREA,PREC,H,KER)
      IF(KER.NE.0)CALL ERRMESS('DELTAE',KER+10)
      AREA2=AREA+AREA2
*     WRITE(6,8)A,Z,AREA,AREA2
* 8   FORMAT(' ',2F10.5,2D20.9)
      IF(Z.GT.B)THEN
          DEL=3.0/GAM
          IF(AREA.LT.AREA2*1.0D-6)GOTO 29
          RELERR=0.1*EPS*AREA2/AREA
      ENDIF
      IF(Z.LT.100.0)GOTO 20
  29  CONTINUE
COMBINE THE TWO AND STICK IN SOME NUMERICAL FACTORS.
      A2BIS = (AREA2/GAMSQ)**2 * ALF**6 / TWOPI
*     WRITE(6,37)AREA1,A2BIS
* 37  FORMAT(' DELTAE I1,I2:',2D14.5)
      DELTAE  = (AREA1-A2BIS) * 16.0  /PI
      END
C ---------------------------- AUXILIARY FUNCTION ROUTINES FOR DELTAE
      FUNCTION GK0(U)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON /COMBES/ ALF,GAM,B,R,XX,YY,AA
      IF(U.EQ.0)THEN
          GK0=0.0
          RETURN
      ENDIF
      CALL DBESK(U,0,BK,KER)
      IF(KER.NE.0)CALL ERRMESS('GK0',KER)
      GK0=BK*U*BKGONIO(GAM*U)
      END
      FUNCTION GK0SQ(U)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON /COMBES/ ALF,GAM,B,R,XX,YY,AA
      IF(U.EQ.0)THEN
          GK0SQ=0.0
          RETURN
      ENDIF
      CALL DBESK(U,0,BK,KER)
      IF(KER.NE.0)CALL ERRMESS('GK0SQ',KER)
      GK0SQ=BK*BK*U*BKGONIO(ALF*U)
      END
C ------------------------------------------------ THE INNER INTEGRAL
      FUNCTION BKGONIO(R1)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
COMPUTE INTEGRAL FROM 0 TO PI OVER PHI OF
C RHO*K1(RHO), WITH RHO=SQRT(X**2 + Y**2 - 2*X*Y*COS(PHI))
C WHERE K1 IS A MODIFIED BESSEL FUNCTION.
C USES EXP(-AA*PHI) AS INTEGRATION VARIABLE, WITH AA=0.3*MIN(X,Y).
      PARAMETER(PI=0.314159265358979D+01)
      EXTERNAL XK1
      COMMON /COMBES/ ALF,GAM,B,R,XX,YY,AA
      XX=R1
      RMIN=ABS(XX-YY)
      RMAX=XX+YY
      AA=0.15*(RMAX-RMIN)
      A0=EXP(-PI*AA)
      DELU=0.25*(1.0-A0)
      TEGRAL=0.0
      PREC=DELU/1200.0
      RELERR=3.0D-5
      DO 3 JP=0,3
      A=A0+JP*DELU
      Z=A+DELU
      IF(Z.GT.1.0)Z=1.0
      CALL DROMBERG(XK1,A,Z,RELERR,AREA,PREC,H,KER)
      IF(KER.NE.0)THEN
          CALL ERRMESS('BKGONIO',KER)
          WRITE(6,184)XX,YY,A,Z,RELERR,AREA,TEGRAL,H
 184      FORMAT(' XX,YY,A,Z:',4D14.6/' R,DI,I,H: ',4D14.6)
      CALL DROMBERG(XK1,A,Z,RELERR,AREA,PREC,H,KER)
          WRITE(6,184)XX,YY,A,Z,RELERR,AREA,TEGRAL,H
      ENDIF
      TEGRAL=TEGRAL+AREA
  3   CONTINUE
      BKGONIO=TEGRAL
      END
      FUNCTION XK1(UU)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      COMMON /COMBES/ ALF,GAM,B,R,XX,YY,AA
      PHI=-LOG(UU)/AA
      Z=SQRT( (XX-YY)**2 + 4.0*XX*YY* SIN(0.5*PHI)**2 )
      IF(Z.EQ.0)Z=1.0D-20
      IF(Z.GE.170.0)THEN
          BK=0
      ELSE
          CALL DBESK(Z,1,BK,IER)
          IF(IER.NE.0)CALL ERRMESS('XK1',IER)
      ENDIF
      XK1=BK*Z/(AA*UU)
      END
C ------------------------------------------------ SUBROUTINE DROMBERG
          SUBROUTINE DROMBERG(F,A,B,EPS,OPP,PREC,H,KERR)
C DWOMBERG AND DROMBERG ARE IDENTICAL TWINS.
C FOR THE DOUBLE INTEGRAL WE NEED TWO COPIES OF THIS ROUTINE.
C   PERFORMS A FIRST ORDER ROMBERG  INTEGRATION OF A FUNCTION
C   F OVER THE INTERVAL (A,B). EPS IS THE RELATIVE CONVERGENCE
C   CRITERIUM. F IS TO BE DECLARED EXTERNAL IN THE CALLING
C   PROGRAM UNIT.
C    OPP  = RESULT OF  INTEGRATION
C    PREC = MINIMUM INTERVAL LENGTH ALLOWED (ASSUMED POSITIVE)
C    H    = MINIMUM INTERVAL LENGTH USED
C    KERR = 0   NO ERRORS
C         = 1   ABS(A-B)< PREC AS INPUT VALUE
C         = 2   INTERVAL LENGTH < PREC BEFORE CONVERGENCE
C
      IMPLICIT DOUBLE PRECISION (A-Z)
      INTEGER KERR
      EXTERNAL F
          KERR=0
          JNEW=0.0
          H=B-A
       IF(ABS(H).LT.PREC)THEN
             KERR=1
             GOTO10
       ENDIF
          IOLD=(F(A)+F(B))*H*0.5
          JOLD=IOLD
  1       SUM=0.0
          X=A+H*0.5
  2       SUM=SUM+F(X)
          X=X+H
       IF(ABS(X-A).LT.ABS(B-A))GOTO2
          H=H*0.5
          INEW=H*SUM+IOLD*0.5
          JNEW=(4.0*INEW-IOLD)/3.0
       IF(ABS(H).LT.PREC)THEN
          KERR=2
             GOTO 10
       ENDIF
       IOLD=INEW
       DIFF=ABS(JNEW-JOLD)
        JOLD=JNEW
      IF(DIFF.GE.EPS*JNEW) GOTO 1
  10   OPP=JNEW
       RETURN
       END
C ------------------------------------------------ SUBROUTINE DWOMBERG
          SUBROUTINE DWOMBERG(F,A,B,EPS,OPP,PREC,H,KERR)
C DWOMBERG AND DROMBERG ARE IDENTICAL TWINS.
C FOR THE DOUBLE INTEGRAL WE NEED TWO COPIES OF THIS ROUTINE.
C   PERFORMS A FIRST ORDER ROMBERG  INTEGRATION OF A FUNCTION
C   F OVER THE INTERVAL (A,B). EPS IS THE RELATIVE CONVERGENCE
C   CRITERIUM. F IS TO BE DECLARED EXTERNAL IN THE CALLING
C   PROGRAM UNIT.
C    OPP  = RESULT OF  INTEGRATION
C    PREC = MINIMUM INTERVAL LENGTH ALLOWED (ASSUMED POSITIVE)
C    H    = MINIMUM INTERVAL LENGTH USED
C    KERR = 0   NO ERRORS
C         = 1   ABS(A-B)< PREC AS INPUT VALUE
C         = 2   INTERVAL LENGTH < PREC BEFORE CONVERGENCE
C
      IMPLICIT DOUBLE PRECISION (A-Z)
      INTEGER KERR
      EXTERNAL F
          KERR=0
          JNEW=0.0
          H=B-A
       IF(ABS(H).LT.PREC)THEN
             KERR=1
             GOTO10
       ENDIF
          IOLD=(F(A)+F(B))*H*0.5
          JOLD=IOLD
  1       SUM=0.0
          X=A+H*0.5
  2       SUM=SUM+F(X)
          X=X+H
       IF(ABS(X-A).LT.ABS(B-A))GOTO2
          H=H*0.5
          INEW=H*SUM+IOLD*0.5
          JNEW=(4.0*INEW-IOLD)/3.0
       IF(ABS(H).LT.PREC)THEN
          KERR=2
             GOTO 10
       ENDIF
       IOLD=INEW
       DIFF=ABS(JNEW-JOLD)
        JOLD=JNEW
      IF(DIFF.GE.EPS*JNEW) GOTO 1
  10   OPP=JNEW
       RETURN
       END
C ----------------------------------------------- SUBROUTINE PRINTIME
      SUBROUTINE PRINTIME
      CHARACTER*20 TIME
      CALL SYSTIM(TIME)
      WRITE(6,6)TIME
   6  FORMAT(1X,A17)
      END
C        SUBROUTINE DBESK
C
C           COMPUTE THE K BESSEL FUNCTION FOR A GIVEN ARGUMENT AND ORDER
C
C        USAGE
C           CALL DBESK(X,N,BK,IER)
C
C        DESCRIPTION OF PARAMETERS
C           X  -THE ARGUMENT OF THE K BESSEL FUNCTION DESIRED
C           N  -THE ORDER OF THE K BESSEL FUNCTION DESIRED
C           BK -THE RESULTANT K BESSEL FUNCTION
C           IER-RESULTANT ERROR CODE WHERE
C              IER=0  NO ERROR
C              IER=1  N IS NEGATIVE
C              IER=2  X IS ZERO OR NEGATIVE
C              IER=3  IF X.GT.170, BOTH K0(X) AND K1(X) ARE ZERO
C                     AND RECURRENCE NO LONGER MEANINGFUL
C              IER=4  BK .GT. 10**72
C
C        REMARKS
C           N MUST BE GREATER THAN OR EQUAL TO ZERO
C           THERE ARE NO RESTRICTIONS ON THE ORDER, BUT FOR A GIVEN X,
C           KN(X) TENDS TO INFINITY AS N TENDS TO INFINITY AND A LARGE
C           ORDER MAY CAUSE OVERFLOW.
C           FOR ARGUMENTS LESS THAN OR EQUAL TO 10 , THE ORDER
C           REQUESTED SHOULD BE LESS THAN OR EQUAL TO 60.
C
C        SUBROUTINES AND FUNCTION SUBPROGRAMS REQUIRED
C           NONE
C
C        METHOD
C           COMPUTES ZERO ORDER AND FIRST ORDER BESSEL FUNCTIONS USING
C           SERIES APPROXIMATIONS AND THEN COMPUTES N TH ORDER FUNCTION
C           USING RECURRENCE RELATION.
C           RECURRENCE RELATION AND POLYNOMIAL APPROXIMATION TECHNIQUE
C           AS DESCRIBED BY A.J.M.HITCHCOCK,'POLYNOMIAL APPROXIMATIONS
C           TO BESSEL FUNCTIONS OF ORDER ZERO AND ONE AND TO RELATED
C           FUNCTIONS', M.T.A.C., V.11,1957,PP.86-88, AND G.N. WATSON,
C           'A TREATISE ON THE THEORY OF BESSEL FUNCTIONS', CAMBRIDGE
C           UNIVERSITY PRESS, 1958, P. 62
C
C     ..................................................................
C
      SUBROUTINE DBESK(X,N,BK,IER)
      IMPLICIT DOUBLE PRECISION(A-H,O-Z)
      DIMENSION T(12)
      BK=.0
      IER=0
      IF(N.LT.0)IER=1
      IF(X.LE.0.0)IER=2
      IF(X.GT.170.0)IER=3
      IF(IER.NE.0)RETURN
      IF(X.LE.1.0)GOTO 36
      A=EXP(-X)
      B=1./X
      C=SQRT(B)
      T(1)=B
      DO 26 L=2,12
   26 T(L)=T(L-1)*B
      IF(N.EQ.1)GOTO 29
C
C     COMPUTE KO USING POLYNOMIAL APPROXIMATION
C
      G0=A*(1.25331414D0-.1566642D0*T(1)+.08811128D0*T(2)-.09139095D0
     2*T(3)+.1344596D0*T(4)-.2299850D0*T(5)+.3792410D0*T(6)-.5247277D0*
     3T(7)+.5575368D0*T(8)-.4262633D0*T(9)+.2184518D0*T(10)-.06680977D0*
     4T(11)+.009189383D0*T(12))*C
      IF(N.NE.0)GOTO 29
      BK=G0
      RETURN
C
C     COMPUTE K1 USING POLYNOMIAL APPROXIMATION
C
   29 G1=A*(1.25331414D0+.4699927D0*T(1)-.1468583D0*T(2)+.1280427D0*T(3)
     2-.1736432D0*T(4)+.2847618D0*T(5)-.4594342D0*T(6)+.6283381D0*T(7)
     3-.6632295D0*T(8)+.5050239D0*T(9)-.2581304D0*T(10)+.07880001D0
     4*T(11)-.01082418D0*T(12))*C
      IF(N.NE.1)GOTO 31
      BK=G1
      RETURN
C
C     FROM KO,K1 COMPUTE KN USING RECURRENCE RELATION
C
   31 DO 35 J=2,N
      GJ=2.*(FLOAT(J)-1.)*G1/X+G0
      IF(GJ.GT.1.0D72)THEN
          IER=4
          GOTO 34
      ENDIF
      G0=G1
   35 G1=GJ
   34 BK=GJ
      RETURN
   36 B=X/2.
      A=0.57721566490153D0+LOG(B)
      C=B*B
      IF(N.EQ.1)GOTO 43
C
C     COMPUTE KO USING SERIES EXPANSION
C
      G0=-A
      X2J=1.
      FACT=1.
      HJ=.0
C FOLLOWING LOOP HAD A FIXED NUMBER (6) OF ITERATIONS.
C MODIFIED TO AVOID UNDERFLOW FOR VERY SMALL ARGUMENTS. 10/11/81.
C A SIMILAR CHANGE WAS MADE IN THE CALCULATION OF K1.
      QJ=0.0
  40  QJ=QJ+1.0
      RJ=1.0/QJ
      X2J=X2J*C
      FACT=FACT*RJ*RJ
      HJ=HJ+RJ
      G0=G0+X2J*FACT*(HJ-A)
      IF(X2J.LT.1.0D-4)GOTO 41
      GO TO 40
   41 IF(N.NE.0)GOTO 43
      BK=G0
      RETURN
C
C     COMPUTE K1 USING SERIES EXPANSION
C
   43 X2J=B
      FACT=1.
      HJ=1.
      G1=1./X+X2J*(.5+A-HJ)
      FLJ=2.0
   50 X2J=X2J*C
      RJ=1./FLJ
      FACT=FACT*RJ*RJ
      HJ=HJ+RJ
      G1=G1+X2J*FACT*(.5+(A-HJ)*FLJ)
      IF(X2J.LT.1.0D-4)GOTO 51
      FLJ=FLJ+1.0
      GOTO 50
   51 IF(N.NE.1)GOTO 31
      BK=G1
      END

      SUBROUTINE ERRMESS(TEXT,NUMBER)
      CHARACTER*(*) TEXT
      WRITE(6,1)TEXT,NUMBER
  1   FORMAT(' !! ',A,' ERROR NUMBER',I6)
      END

      SUBROUTINE INREAL1(ARG,PROMPT)
C usage:  CALL INREAL1(ARG,PROMPT)
      REAL ARG
      CHARACTER*(*) PROMPT
  1   FORMAT(1X,A,A1,G12.5)
  2   FORMAT(1X,A,':',$)
      WRITE(6,2)PROMPT
      READ (5,*)ARG
      WRITE(6,1) PROMPT,':',ARG
      END
